"""The container module provides the Container class.

See https://argoproj.github.io/argo-workflows/workflow-concepts/#container
for more on containers.
"""
from __future__ import annotations

from typing import List, Optional

from hera.workflows._mixins import (
    CallableTemplateMixin,
    ContainerMixin,
    EnvIOMixin,
    ResourceMixin,
    TemplateMixin,
    VolumeMountMixin,
)
from hera.workflows.models import (
    Container as _ModelContainer,
    Lifecycle,
    SecurityContext,
    Template as _ModelTemplate,
)


class Container(
    EnvIOMixin,
    ContainerMixin,
    TemplateMixin,
    ResourceMixin,
    VolumeMountMixin,
    CallableTemplateMixin,
):
    """The Container template type defines a container to run on Argo."""

    args: Optional[List[str]] = None
    command: Optional[List[str]] = None
    lifecycle: Optional[Lifecycle] = None
    security_context: Optional[SecurityContext] = None
    working_dir: Optional[str] = None

    def _build_container(self) -> _ModelContainer:
        return _ModelContainer(
            args=self.args,
            command=self.command,
            env=self._build_env(),
            env_from=self._build_env_from(),
            image=self.image,
            image_pull_policy=self._build_image_pull_policy(),
            lifecycle=self.lifecycle,
            liveness_probe=self.liveness_probe,
            ports=self.ports,
            readiness_probe=self.readiness_probe,
            resources=self._build_resources(),
            security_context=self.security_context,
            startup_probe=self.startup_probe,
            stdin=self.stdin,
            stdin_once=self.stdin_once,
            termination_message_path=self.termination_message_path,
            termination_message_policy=self.termination_message_policy,
            tty=self.tty,
            volume_devices=self.volume_devices,
            volume_mounts=self._build_volume_mounts(),
            working_dir=self.working_dir,
        )

    def _build_template(self) -> _ModelTemplate:
        return _ModelTemplate(
            active_deadline_seconds=self.active_deadline_seconds,
            affinity=self.affinity,
            archive_location=self.archive_location,
            automount_service_account_token=self.automount_service_account_token,
            container=self._build_container(),
            daemon=self.daemon,
            executor=self.executor,
            fail_fast=self.fail_fast,
            host_aliases=self.host_aliases,
            init_containers=self.init_containers,
            inputs=self._build_inputs(),
            memoize=self.memoize,
            metadata=self._build_metadata(),
            metrics=self.metrics,
            name=self.name,
            node_selector=self.node_selector,
            outputs=self._build_outputs(),
            plugin=self.plugin,
            pod_spec_patch=self.pod_spec_patch,
            priority=self.priority,
            priority_class_name=self.priority_class_name,
            resource=self._build_resources(),
            retry_strategy=self.retry_strategy,
            scheduler_name=self.scheduler_name,
            security_context=self.pod_security_context,
            service_account_name=self.service_account_name,
            sidecars=self.sidecars,
            synchronization=self.synchronization,
            timeout=self.timeout,
            tolerations=self.tolerations,
            volumes=self._build_volumes(),
        )


__all__ = ["Container"]



"""Internal module to help transpile spring functions to expr.

See https://argoproj.github.io/argo-workflows/variables/#examples and
http://masterminds.github.io/sprig/ for more information.

Available publicly via the `sprig.<function>` variable.
"""

from typing import Any

from hera.expr._node import Callable


class _Method:
    def __init__(self, name: str):
        self.name = name

    def __call__(self, *args: Any, **_: Any) -> Any:
        return Callable(self.name, *args)


class Sprig:
    """Supports transpiling spring function calls."""

    def __getattr__(self, name: str) -> _Method:
        return _Method(f"sprig.{name}")



# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/argoproj/argo-workflows/v3.4.4/api/openapi-spec/swagger.json

from __future__ import annotations

from typing import Optional

from pydantic import Field

from hera.shared._base_model import BaseModel

from .io.argoproj.events import v1alpha1
from .io.k8s.apimachinery.pkg.apis.meta import v1


class DeleteSensorResponse(BaseModel):
    pass


class LogEntry(BaseModel):
    dependency_name: Optional[str] = Field(None, alias="dependencyName", title="optional - trigger dependency name")
    event_context: Optional[str] = Field(None, alias="eventContext", title="optional - Cloud Event context")
    level: Optional[str] = None
    msg: Optional[str] = None
    namespace: Optional[str] = None
    sensor_name: Optional[str] = Field(None, alias="sensorName")
    time: Optional[v1.Time] = None
    trigger_name: Optional[str] = Field(None, alias="triggerName", title="optional - any trigger name")


class CreateSensorRequest(BaseModel):
    create_options: Optional[v1.CreateOptions] = Field(None, alias="createOptions")
    namespace: Optional[str] = None
    sensor: Optional[v1alpha1.Sensor] = None


class SensorWatchEvent(BaseModel):
    object: Optional[v1alpha1.Sensor] = None
    type: Optional[str] = None


class UpdateSensorRequest(BaseModel):
    name: Optional[str] = None
    namespace: Optional[str] = None
    sensor: Optional[v1alpha1.Sensor] = None



import os
from typing import Optional, cast

import requests

from hera.events.models import (
    CreateEventSourceRequest,
    CreateSensorRequest,
    DeleteSensorResponse,
    Event,
    EventResponse,
    EventSource,
    EventSourceDeletedResponse,
    EventSourceList,
    EventsourceLogEntry,
    EventSourceWatchEvent,
    GetUserInfoResponse,
    InfoResponse,
    Item,
    Sensor,
    SensorList,
    SensorLogEntry,
    SensorWatchEvent,
    UpdateEventSourceRequest,
    UpdateSensorRequest,
    Version,
)
from hera.shared import global_config


class EventsService:
    def __init__(
        self,
        host: Optional[str] = None,
        verify_ssl: Optional[bool] = None,
        token: Optional[str] = None,
        namespace: Optional[str] = None,
    ):
        self.host = cast(str, host or global_config.host)
        self.verify_ssl = verify_ssl or global_config.verify_ssl
        self.token = token or global_config.token
        self.namespace = namespace or global_config.namespace

    def list_event_sources(
        self,
        namespace: str,
        label_selector: Optional[str] = None,
        field_selector: Optional[str] = None,
        watch: Optional[bool] = None,
        allow_watch_bookmarks: Optional[bool] = None,
        resource_version: Optional[str] = None,
        resource_version_match: Optional[str] = None,
        timeout_seconds: Optional[str] = None,
        limit: Optional[str] = None,
        continue_: Optional[str] = None,
    ) -> EventSourceList:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/event-sources/{namespace}").format(namespace=namespace),
            params={
                "listOptions.labelSelector": label_selector,
                "listOptions.fieldSelector": field_selector,
                "listOptions.watch": watch,
                "listOptions.allowWatchBookmarks": allow_watch_bookmarks,
                "listOptions.resourceVersion": resource_version,
                "listOptions.resourceVersionMatch": resource_version_match,
                "listOptions.timeoutSeconds": timeout_seconds,
                "listOptions.limit": limit,
                "listOptions.continue": continue_,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSourceList(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def create_event_source(self, namespace: str, req: CreateEventSourceRequest) -> EventSource:
        resp = requests.post(
            url=os.path.join(self.host, "api/v1/event-sources/{namespace}").format(namespace=namespace),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=req.json(
                exclude_none=True, by_alias=True, skip_defaults=True, exclude_unset=True, exclude_defaults=True
            ),
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSource(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_event_source(self, namespace: str, name: str) -> EventSource:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/event-sources/{namespace}/{name}").format(
                namespace=namespace, name=name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSource(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def update_event_source(self, namespace: str, name: str, req: UpdateEventSourceRequest) -> EventSource:
        resp = requests.put(
            url=os.path.join(self.host, "api/v1/event-sources/{namespace}/{name}").format(
                namespace=namespace, name=name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=req.json(
                exclude_none=True, by_alias=True, skip_defaults=True, exclude_unset=True, exclude_defaults=True
            ),
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSource(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def delete_event_source(
        self,
        namespace: str,
        name: str,
        grace_period_seconds: Optional[str] = None,
        uid: Optional[str] = None,
        resource_version: Optional[str] = None,
        orphan_dependents: Optional[bool] = None,
        propagation_policy: Optional[str] = None,
        dry_run: Optional[list] = None,
    ) -> EventSourceDeletedResponse:
        resp = requests.delete(
            url=os.path.join(self.host, "api/v1/event-sources/{namespace}/{name}").format(
                namespace=namespace, name=name
            ),
            params={
                "deleteOptions.gracePeriodSeconds": grace_period_seconds,
                "deleteOptions.preconditions.uid": uid,
                "deleteOptions.preconditions.resourceVersion": resource_version,
                "deleteOptions.orphanDependents": orphan_dependents,
                "deleteOptions.propagationPolicy": propagation_policy,
                "deleteOptions.dryRun": dry_run,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSourceDeletedResponse()
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def receive_event(self, namespace: str, discriminator: str, req: Item) -> EventResponse:
        resp = requests.post(
            url=os.path.join(self.host, "api/v1/events/{namespace}/{discriminator}").format(
                namespace=namespace, discriminator=discriminator
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=req.json(
                exclude_none=True, by_alias=True, skip_defaults=True, exclude_unset=True, exclude_defaults=True
            ),
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventResponse()
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_info(self) -> InfoResponse:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/info"),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return InfoResponse()
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def list_sensors(
        self,
        namespace: str,
        label_selector: Optional[str] = None,
        field_selector: Optional[str] = None,
        watch: Optional[bool] = None,
        allow_watch_bookmarks: Optional[bool] = None,
        resource_version: Optional[str] = None,
        resource_version_match: Optional[str] = None,
        timeout_seconds: Optional[str] = None,
        limit: Optional[str] = None,
        continue_: Optional[str] = None,
    ) -> SensorList:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/sensors/{namespace}").format(namespace=namespace),
            params={
                "listOptions.labelSelector": label_selector,
                "listOptions.fieldSelector": field_selector,
                "listOptions.watch": watch,
                "listOptions.allowWatchBookmarks": allow_watch_bookmarks,
                "listOptions.resourceVersion": resource_version,
                "listOptions.resourceVersionMatch": resource_version_match,
                "listOptions.timeoutSeconds": timeout_seconds,
                "listOptions.limit": limit,
                "listOptions.continue": continue_,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return SensorList(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def create_sensor(self, namespace: str, req: CreateSensorRequest) -> Sensor:
        resp = requests.post(
            url=os.path.join(self.host, "api/v1/sensors/{namespace}").format(namespace=namespace),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=req.json(
                exclude_none=True, by_alias=True, skip_defaults=True, exclude_unset=True, exclude_defaults=True
            ),
            verify=self.verify_ssl,
        )

        if resp.ok:
            return Sensor(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_sensor(self, namespace: str, name: str, resource_version: Optional[str] = None) -> Sensor:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/sensors/{namespace}/{name}").format(namespace=namespace, name=name),
            params={"getOptions.resourceVersion": resource_version},
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return Sensor(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def update_sensor(self, namespace: str, name: str, req: UpdateSensorRequest) -> Sensor:
        resp = requests.put(
            url=os.path.join(self.host, "api/v1/sensors/{namespace}/{name}").format(namespace=namespace, name=name),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=req.json(
                exclude_none=True, by_alias=True, skip_defaults=True, exclude_unset=True, exclude_defaults=True
            ),
            verify=self.verify_ssl,
        )

        if resp.ok:
            return Sensor(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def delete_sensor(
        self,
        namespace: str,
        name: str,
        grace_period_seconds: Optional[str] = None,
        uid: Optional[str] = None,
        resource_version: Optional[str] = None,
        orphan_dependents: Optional[bool] = None,
        propagation_policy: Optional[str] = None,
        dry_run: Optional[list] = None,
    ) -> DeleteSensorResponse:
        resp = requests.delete(
            url=os.path.join(self.host, "api/v1/sensors/{namespace}/{name}").format(namespace=namespace, name=name),
            params={
                "deleteOptions.gracePeriodSeconds": grace_period_seconds,
                "deleteOptions.preconditions.uid": uid,
                "deleteOptions.preconditions.resourceVersion": resource_version,
                "deleteOptions.orphanDependents": orphan_dependents,
                "deleteOptions.propagationPolicy": propagation_policy,
                "deleteOptions.dryRun": dry_run,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return DeleteSensorResponse()
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def watch_event_sources(
        self,
        namespace: str,
        label_selector: Optional[str] = None,
        field_selector: Optional[str] = None,
        watch: Optional[bool] = None,
        allow_watch_bookmarks: Optional[bool] = None,
        resource_version: Optional[str] = None,
        resource_version_match: Optional[str] = None,
        timeout_seconds: Optional[str] = None,
        limit: Optional[str] = None,
        continue_: Optional[str] = None,
    ) -> EventSourceWatchEvent:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/stream/event-sources/{namespace}").format(namespace=namespace),
            params={
                "listOptions.labelSelector": label_selector,
                "listOptions.fieldSelector": field_selector,
                "listOptions.watch": watch,
                "listOptions.allowWatchBookmarks": allow_watch_bookmarks,
                "listOptions.resourceVersion": resource_version,
                "listOptions.resourceVersionMatch": resource_version_match,
                "listOptions.timeoutSeconds": timeout_seconds,
                "listOptions.limit": limit,
                "listOptions.continue": continue_,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventSourceWatchEvent(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def event_sources_logs(
        self,
        namespace: str,
        name: Optional[str] = None,
        event_source_type: Optional[str] = None,
        event_name: Optional[str] = None,
        grep: Optional[str] = None,
        container: Optional[str] = None,
        follow: Optional[bool] = None,
        previous: Optional[bool] = None,
        since_seconds: Optional[str] = None,
        seconds: Optional[str] = None,
        nanos: Optional[int] = None,
        timestamps: Optional[bool] = None,
        tail_lines: Optional[str] = None,
        limit_bytes: Optional[str] = None,
        insecure_skip_tls_verify_backend: Optional[bool] = None,
    ) -> EventsourceLogEntry:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/stream/event-sources/{namespace}/logs").format(namespace=namespace),
            params={
                "name": name,
                "eventSourceType": event_source_type,
                "eventName": event_name,
                "grep": grep,
                "podLogOptions.container": container,
                "podLogOptions.follow": follow,
                "podLogOptions.previous": previous,
                "podLogOptions.sinceSeconds": since_seconds,
                "podLogOptions.sinceTime.seconds": seconds,
                "podLogOptions.sinceTime.nanos": nanos,
                "podLogOptions.timestamps": timestamps,
                "podLogOptions.tailLines": tail_lines,
                "podLogOptions.limitBytes": limit_bytes,
                "podLogOptions.insecureSkipTLSVerifyBackend": insecure_skip_tls_verify_backend,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return EventsourceLogEntry(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def watch_events(
        self,
        namespace: str,
        label_selector: Optional[str] = None,
        field_selector: Optional[str] = None,
        watch: Optional[bool] = None,
        allow_watch_bookmarks: Optional[bool] = None,
        resource_version: Optional[str] = None,
        resource_version_match: Optional[str] = None,
        timeout_seconds: Optional[str] = None,
        limit: Optional[str] = None,
        continue_: Optional[str] = None,
    ) -> Event:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/stream/events/{namespace}").format(namespace=namespace),
            params={
                "listOptions.labelSelector": label_selector,
                "listOptions.fieldSelector": field_selector,
                "listOptions.watch": watch,
                "listOptions.allowWatchBookmarks": allow_watch_bookmarks,
                "listOptions.resourceVersion": resource_version,
                "listOptions.resourceVersionMatch": resource_version_match,
                "listOptions.timeoutSeconds": timeout_seconds,
                "listOptions.limit": limit,
                "listOptions.continue": continue_,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return Event(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def watch_sensors(
        self,
        namespace: str,
        label_selector: Optional[str] = None,
        field_selector: Optional[str] = None,
        watch: Optional[bool] = None,
        allow_watch_bookmarks: Optional[bool] = None,
        resource_version: Optional[str] = None,
        resource_version_match: Optional[str] = None,
        timeout_seconds: Optional[str] = None,
        limit: Optional[str] = None,
        continue_: Optional[str] = None,
    ) -> SensorWatchEvent:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/stream/sensors/{namespace}").format(namespace=namespace),
            params={
                "listOptions.labelSelector": label_selector,
                "listOptions.fieldSelector": field_selector,
                "listOptions.watch": watch,
                "listOptions.allowWatchBookmarks": allow_watch_bookmarks,
                "listOptions.resourceVersion": resource_version,
                "listOptions.resourceVersionMatch": resource_version_match,
                "listOptions.timeoutSeconds": timeout_seconds,
                "listOptions.limit": limit,
                "listOptions.continue": continue_,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return SensorWatchEvent(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def sensors_logs(
        self,
        namespace: str,
        name: Optional[str] = None,
        trigger_name: Optional[str] = None,
        grep: Optional[str] = None,
        container: Optional[str] = None,
        follow: Optional[bool] = None,
        previous: Optional[bool] = None,
        since_seconds: Optional[str] = None,
        seconds: Optional[str] = None,
        nanos: Optional[int] = None,
        timestamps: Optional[bool] = None,
        tail_lines: Optional[str] = None,
        limit_bytes: Optional[str] = None,
        insecure_skip_tls_verify_backend: Optional[bool] = None,
    ) -> SensorLogEntry:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/stream/sensors/{namespace}/logs").format(namespace=namespace),
            params={
                "name": name,
                "triggerName": trigger_name,
                "grep": grep,
                "podLogOptions.container": container,
                "podLogOptions.follow": follow,
                "podLogOptions.previous": previous,
                "podLogOptions.sinceSeconds": since_seconds,
                "podLogOptions.sinceTime.seconds": seconds,
                "podLogOptions.sinceTime.nanos": nanos,
                "podLogOptions.timestamps": timestamps,
                "podLogOptions.tailLines": tail_lines,
                "podLogOptions.limitBytes": limit_bytes,
                "podLogOptions.insecureSkipTLSVerifyBackend": insecure_skip_tls_verify_backend,
            },
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return SensorLogEntry(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_user_info(self) -> GetUserInfoResponse:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/userinfo"),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return GetUserInfoResponse()
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_version(self) -> Version:
        resp = requests.get(
            url=os.path.join(self.host, "api/v1/version"),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return Version(**resp.json())
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_artifact_file(
        self,
        namespace: str,
        id_discriminator: str,
        id_: str,
        node_id: str,
        artifact_name: str,
        artifact_discriminator: str,
    ) -> str:
        """Get an artifact."""
        resp = requests.get(
            url=os.path.join(
                self.host,
                "artifact-files/{namespace}/{idDiscriminator}/{id}/{nodeId}/{artifactDiscriminator}/{artifactName}",
            ).format(
                namespace=namespace,
                idDiscriminator=id_discriminator,
                id=id_,
                nodeId=node_id,
                artifactName=artifact_name,
                artifactDiscriminator=artifact_discriminator,
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return str(resp.content)
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_output_artifact_by_uid(self, uid: str, node_id: str, artifact_name: str) -> str:
        """Get an output artifact by UID."""
        resp = requests.get(
            url=os.path.join(self.host, "artifacts-by-uid/{uid}/{nodeId}/{artifactName}").format(
                uid=uid, nodeId=node_id, artifactName=artifact_name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return str(resp.content)
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_output_artifact(self, namespace: str, name: str, node_id: str, artifact_name: str) -> str:
        """Get an output artifact."""
        resp = requests.get(
            url=os.path.join(self.host, "artifacts/{namespace}/{name}/{nodeId}/{artifactName}").format(
                namespace=namespace, name=name, nodeId=node_id, artifactName=artifact_name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return str(resp.content)
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_input_artifact_by_uid(self, uid: str, node_id: str, artifact_name: str) -> str:
        """Get an input artifact by UID."""
        resp = requests.get(
            url=os.path.join(self.host, "input-artifacts-by-uid/{uid}/{nodeId}/{artifactName}").format(
                uid=uid, nodeId=node_id, artifactName=artifact_name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return str(resp.content)
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")

    def get_input_artifact(self, namespace: str, name: str, node_id: str, artifact_name: str) -> str:
        """Get an input artifact."""
        resp = requests.get(
            url=os.path.join(self.host, "input-artifacts/{namespace}/{name}/{nodeId}/{artifactName}").format(
                namespace=namespace, name=name, nodeId=node_id, artifactName=artifact_name
            ),
            params=None,
            headers={"Authorization": f"Bearer {self.token}"},
            data=None,
            verify=self.verify_ssl,
        )

        if resp.ok:
            return str(resp.content)
        else:
            raise Exception(f"Server returned status code {resp.status_code} with error: {resp.json()}")


__all__ = ["EventsService"]



"""The workflow module provides the Workflow class

See https://argoproj.github.io/argo-workflows/workflow-concepts/#the-workflow
for more on Workflows.
"""
from __future__ import annotations

from types import ModuleType
from typing import Any, Dict, List, Optional, Union

from pydantic import validator
from typing_extensions import get_args

from hera.shared import global_config
from hera.workflows._mixins import ArgumentsMixin, ContextMixin, HookMixin
from hera.workflows.exceptions import InvalidType
from hera.workflows.models import (
    Affinity,
    ArtifactGC,
    ArtifactRepositoryRef,
    ExecutorConfig,
    HostAlias,
    LifecycleHook,
    LocalObjectReference,
    ManagedFieldsEntry,
    Metadata,
    Metrics,
    ObjectMeta,
    OwnerReference,
    PersistentVolumeClaim,
    PodDisruptionBudgetSpec,
    PodDNSConfig,
    PodGC,
    PodSecurityContext,
    RetryStrategy,
    Synchronization,
    Template,
    Time,
    Toleration,
    TTLStrategy,
    Volume,
    VolumeClaimGC,
    Workflow as _ModelWorkflow,
    WorkflowCreateRequest,
    WorkflowLintRequest,
    WorkflowMetadata,
    WorkflowSpec as _ModelWorkflowSpec,
    WorkflowStatus,
    WorkflowTemplateRef,
)
from hera.workflows.protocol import Templatable, TTemplate, TWorkflow, VolumeClaimable
from hera.workflows.service import WorkflowsService

_yaml: Optional[ModuleType] = None
try:
    import yaml

    _yaml = yaml
except ImportError:
    _yaml = None

ImagePullSecrets = Optional[Union[LocalObjectReference, List[LocalObjectReference], str, List[str]]]


class Workflow(
    ArgumentsMixin,
    ContextMixin,
    HookMixin,
):
    """The base Workflow class for Hera.

    Workflow implements the contextmanager interface so allows usage of `with`, under which
    any `hera.workflows.protocol.Templatable` object or `hera.workflows.models.Template` object
    instantiated under the context will be added to the Workflow's list of templates.

    Workflows can be created directly on your Argo cluster via `create`. They can also be dumped
    to yaml via `to_yaml` or built according to the Argo schema via `build` to get an OpenAPI model
    object.
    """

    # Workflow fields - https://argoproj.github.io/argo-workflows/fields/#workflow
    api_version: Optional[str] = global_config.api_version
    kind: Optional[str] = None
    status: Optional[WorkflowStatus] = None

    # ObjectMeta fields - https://argoproj.github.io/argo-workflows/fields/#objectmeta
    annotations: Optional[Dict[str, str]] = None
    cluster_name: Optional[str] = None
    creation_timestamp: Optional[Time] = None
    deletion_grace_period_seconds: Optional[int] = None
    deletion_timestamp: Optional[Time] = None
    finalizers: Optional[List[str]] = None
    generate_name: Optional[str] = None
    generation: Optional[int] = None
    labels: Optional[Dict[str, str]] = None
    managed_fields: Optional[List[ManagedFieldsEntry]] = None
    name: Optional[str] = None
    namespace: Optional[str] = None
    owner_references: Optional[List[OwnerReference]] = None
    resource_version: Optional[str] = None
    self_link: Optional[str] = None
    uid: Optional[str] = None

    # WorkflowSpec fields - https://argoproj.github.io/argo-workflows/fields/#workflowspec
    active_deadline_seconds: Optional[int] = None
    affinity: Optional[Affinity] = None
    archive_logs: Optional[bool] = None
    artifact_gc: Optional[ArtifactGC] = None
    artifact_repository_ref: Optional[ArtifactRepositoryRef] = None
    automount_service_account_token: Optional[bool] = None
    dns_config: Optional[PodDNSConfig] = None
    dns_policy: Optional[str] = None
    entrypoint: Optional[str] = None
    executor: Optional[ExecutorConfig] = None
    hooks: Optional[Dict[str, LifecycleHook]] = None
    host_aliases: Optional[List[HostAlias]] = None
    host_network: Optional[bool] = None
    image_pull_secrets: ImagePullSecrets = None
    metrics: Optional[Metrics] = None
    node_selector: Optional[Dict[str, str]] = None
    on_exit: Optional[str] = None
    parallelism: Optional[int] = None
    pod_disruption_budget: Optional[PodDisruptionBudgetSpec] = None
    pod_gc: Optional[PodGC] = None
    pod_metadata: Optional[Metadata] = None
    pod_priority: Optional[int] = None
    pod_priority_class_name: Optional[str] = None
    pod_spec_patch: Optional[str] = None
    priority: Optional[int] = None
    retry_strategy: Optional[RetryStrategy] = None
    scheduler_name: Optional[str] = None
    security_context: Optional[PodSecurityContext] = None
    service_account_name: Optional[str] = None
    shutdown: Optional[str] = None
    suspend: Optional[bool] = None
    synchronization: Optional[Synchronization] = None
    template_defaults: Optional[Template] = None
    templates: List[Union[Template, Templatable]] = []
    tolerations: Optional[List[Toleration]] = None
    ttl_strategy: Optional[TTLStrategy] = None
    volume_claim_gc: Optional[VolumeClaimGC] = None
    volume_claim_templates: Optional[List[PersistentVolumeClaim]] = None
    volumes: Optional[List[Volume]] = None
    workflow_metadata: Optional[WorkflowMetadata] = None
    workflow_template_ref: Optional[WorkflowTemplateRef] = None

    # Hera-specific fields
    workflows_service: Optional[WorkflowsService] = None

    @validator("workflows_service", pre=True, always=True)
    def _set_workflows_service(cls, v):
        if v is None:
            return WorkflowsService()
        return v

    @validator("kind", pre=True, always=True)
    def _set_kind(cls, v):
        if v is None:
            return cls.__name__  # type: ignore
        return v

    @validator("namespace", pre=True, always=True)
    def _set_namespace(cls, v):
        if v is None:
            return global_config.namespace
        return v

    @validator("image_pull_secrets", pre=True, always=True)
    def _set_image_pull_secrets(cls, v):
        if v is None:
            return None

        if isinstance(v, str):
            return [LocalObjectReference(name=v)]
        elif isinstance(v, LocalObjectReference):
            return [v]

        assert isinstance(v, list), (
            "`image_pull_secrets` expected to be either a `str`, a `LocalObjectReferences`, a list of `str`, "
            "or a list of `LocalObjectReferences`"
        )
        result = []
        for secret in v:
            if isinstance(secret, str):
                result.append(LocalObjectReference(name=secret))
            elif isinstance(secret, LocalObjectReference):
                result.append(secret)
        return result

    def build(self) -> TWorkflow:
        """Builds the Workflow and its components into an Argo schema Workflow object."""
        self = self._dispatch_hooks()

        templates = []
        for template in self.templates:
            if isinstance(template, HookMixin):
                template = template._dispatch_hooks()

            if isinstance(template, Templatable):
                templates.append(template._build_template())
            elif isinstance(template, get_args(TTemplate)):
                templates.append(template)
            else:
                raise InvalidType(f"{type(template)} is not a valid template type")

            if isinstance(template, VolumeClaimable):
                claims = template._build_persistent_volume_claims()
                # If there are no claims, continue, nothing to add
                if not claims:
                    continue
                # If there are no volume claim templates, set them to the constructed claims
                elif self.volume_claim_templates is None:
                    self.volume_claim_templates = claims
                else:
                    # otherwise, we need to merge the two lists of volume claim templates. This prioritizes the
                    # already existing volume claim templates under the assumption that the user has already set
                    # a claim template on the workflow intentionally, or the user is sharing the same volumes across
                    # different templates
                    current_volume_claims_map = {}
                    for claim in self.volume_claim_templates:
                        assert claim.metadata is not None, "expected a workflow volume claim with metadata"
                        assert claim.metadata.name is not None, "expected a named workflow volume claim"
                        current_volume_claims_map[claim.metadata.name] = claim

                    new_volume_claims_map = {}
                    for claim in claims:
                        assert claim.metadata is not None, "expected a volume claim with metadata"
                        assert claim.metadata.name is not None, "expected a named volume claim"
                        new_volume_claims_map[claim.metadata.name] = claim

                    for claim_name, claim in new_volume_claims_map.items():
                        if claim_name not in current_volume_claims_map:
                            self.volume_claim_templates.append(claim)

        return _ModelWorkflow(
            api_version=self.api_version,
            kind=self.kind,
            metadata=ObjectMeta(
                annotations=self.annotations,
                cluster_name=self.cluster_name,
                creation_timestamp=self.creation_timestamp,
                deletion_grace_period_seconds=self.deletion_grace_period_seconds,
                deletion_timestamp=self.deletion_timestamp,
                finalizers=self.finalizers,
                generate_name=self.generate_name,
                generation=self.generation,
                labels=self.labels,
                managed_fields=self.managed_fields,
                name=self.name,
                namespace=self.namespace,
                owner_references=self.owner_references,
                resource_version=self.resource_version,
                self_link=self.self_link,
                uid=self.uid,
            ),
            spec=_ModelWorkflowSpec(
                active_deadline_seconds=self.active_deadline_seconds,
                affinity=self.affinity,
                archive_logs=self.archive_logs,
                arguments=self._build_arguments(),
                artifact_gc=self.artifact_gc,
                artifact_repository_ref=self.artifact_repository_ref,
                automount_service_account_token=self.automount_service_account_token,
                dns_config=self.dns_config,
                dns_policy=self.dns_policy,
                entrypoint=self.entrypoint,
                executor=self.executor,
                hooks=self.hooks,
                host_aliases=self.host_aliases,
                host_network=self.host_network,
                image_pull_secrets=self.image_pull_secrets,
                metrics=self.metrics,
                node_selector=self.node_selector,
                on_exit=self.on_exit,
                parallelism=self.parallelism,
                pod_disruption_budget=self.pod_disruption_budget,
                pod_gc=self.pod_gc,
                pod_metadata=self.pod_metadata,
                pod_priority=self.pod_priority,
                pod_priority_class_name=self.pod_priority_class_name,
                pod_spec_patch=self.pod_spec_patch,
                priority=self.priority,
                retry_strategy=self.retry_strategy,
                scheduler_name=self.scheduler_name,
                security_context=self.security_context,
                service_account_name=self.service_account_name,
                shutdown=self.shutdown,
                suspend=self.suspend,
                synchronization=self.synchronization,
                template_defaults=self.template_defaults,
                templates=templates or None,
                tolerations=self.tolerations,
                ttl_strategy=self.ttl_strategy,
                volume_claim_gc=self.volume_claim_gc,
                volume_claim_templates=self.volume_claim_templates,
                volumes=self.volumes,
                workflow_metadata=self.workflow_metadata,
                workflow_template_ref=self.workflow_template_ref,
            ),
            status=self.status,
        )

    def to_dict(self) -> Any:
        """Builds the Workflow as an Argo schema Workflow object and returns it as a dictionary."""
        return self.build().dict(exclude_none=True, by_alias=True)

    def to_yaml(self, *args, **kwargs) -> str:
        """Builds the Workflow as an Argo schema Workflow object and returns it as yaml string."""
        if not _yaml:
            raise ImportError("PyYAML is not installed")
        return _yaml.dump(self.to_dict(), *args, **kwargs)

    def create(self) -> TWorkflow:
        """Creates the Workflow on the Argo cluster."""
        assert self.workflows_service, "workflow service not initialized"
        assert self.namespace, "workflow namespace not defined"
        return self.workflows_service.create_workflow(self.namespace, WorkflowCreateRequest(workflow=self.build()))

    def lint(self) -> TWorkflow:
        """Lints the Workflow using the Argo cluster."""
        assert self.workflows_service, "workflow service not initialized"
        assert self.namespace, "workflow namespace not defined"
        return self.workflows_service.lint_workflow(self.namespace, WorkflowLintRequest(workflow=self.build()))

    def _add_sub(self, node: Any):
        """Adds any objects instantiated under the Workflow context manager that conform to the `Templatable` protocol
        or are Argo schema Template objects to the Workflow's list of templates.
        """
        if not isinstance(node, (Templatable, *get_args(Template))):
            raise InvalidType(type(node))
        self.templates.append(node)


__all__ = ["Workflow"]



from datetime import datetime
from typing import Dict, List, Optional

from hera.shared._base_model import BaseModel as BaseModel

class CreateOptions(BaseModel):
    dry_run: Optional[List[str]]
    field_manager: Optional[str]
    field_validation: Optional[str]

class FieldsV1(BaseModel): ...

class GroupVersionResource(BaseModel):
    group: Optional[str]
    resource: Optional[str]
    version: Optional[str]

class LabelSelectorRequirement(BaseModel):
    key: str
    operator: str
    values: Optional[List[str]]

class ListMeta(BaseModel):
    continue_: Optional[str]
    remaining_item_count: Optional[int]
    resource_version: Optional[str]
    self_link: Optional[str]

class MicroTime(BaseModel):
    __root__: datetime

class OwnerReference(BaseModel):
    api_version: str
    block_owner_deletion: Optional[bool]
    controller: Optional[bool]
    kind: str
    name: str
    uid: str

class StatusCause(BaseModel):
    field: Optional[str]
    message: Optional[str]
    reason: Optional[str]

class Time(BaseModel):
    __root__: datetime

class LabelSelector(BaseModel):
    match_expressions: Optional[List[LabelSelectorRequirement]]
    match_labels: Optional[Dict[str, str]]

class ManagedFieldsEntry(BaseModel):
    api_version: Optional[str]
    fields_type: Optional[str]
    fields_v1: Optional[FieldsV1]
    manager: Optional[str]
    operation: Optional[str]
    subresource: Optional[str]
    time: Optional[Time]

class ObjectMeta(BaseModel):
    annotations: Optional[Dict[str, str]]
    cluster_name: Optional[str]
    creation_timestamp: Optional[Time]
    deletion_grace_period_seconds: Optional[int]
    deletion_timestamp: Optional[Time]
    finalizers: Optional[List[str]]
    generate_name: Optional[str]
    generation: Optional[int]
    labels: Optional[Dict[str, str]]
    managed_fields: Optional[List[ManagedFieldsEntry]]
    name: Optional[str]
    namespace: Optional[str]
    owner_references: Optional[List[OwnerReference]]
    resource_version: Optional[str]
    self_link: Optional[str]
    uid: Optional[str]



"""Auto-generated model classes
"""

# [DO NOT EDIT MANUALLY]

# Auto-generated by Hera via `make workflows-models`
# OpenAPI spec URL: https://raw.githubusercontent.com/argoproj/argo-workflows/v3.4.4/api/openapi-spec/swagger.json

from hera.workflows.models.google.protobuf import Any
from hera.workflows.models.grpc.gateway.runtime import Error, StreamError
from hera.workflows.models.io.argoproj.workflow.v1alpha1 import (
    HTTP,
    Amount,
    ArchivedWorkflowDeletedResponse,
    ArchiveStrategy,
    Arguments,
    ArtGCStatus,
    Artifact,
    ArtifactGC,
    ArtifactGCSpec,
    ArtifactGCStatus,
    ArtifactLocation,
    ArtifactNodeSpec,
    ArtifactoryArtifact,
    ArtifactoryArtifactRepository,
    ArtifactPaths,
    ArtifactRepository,
    ArtifactRepositoryRef,
    ArtifactRepositoryRefStatus,
    ArtifactResult,
    ArtifactResultNodeStatus,
    AzureArtifact,
    AzureArtifactRepository,
    Backoff,
    BasicAuth,
    Cache,
    ClientCertAuth,
    ClusterWorkflowTemplate,
    ClusterWorkflowTemplateCreateRequest,
    ClusterWorkflowTemplateDeleteResponse,
    ClusterWorkflowTemplateLintRequest,
    ClusterWorkflowTemplateList,
    ClusterWorkflowTemplateUpdateRequest,
    CollectEventRequest,
    CollectEventResponse,
    Condition,
    ContainerNode,
    ContainerSetRetryStrategy,
    ContainerSetTemplate,
    ContinueOn,
    Counter,
    CreateCronWorkflowRequest,
    CreateS3BucketOptions,
    CronWorkflow,
    CronWorkflowDeletedResponse,
    CronWorkflowList,
    CronWorkflowResumeRequest,
    CronWorkflowSpec,
    CronWorkflowStatus,
    CronWorkflowSuspendRequest,
    DAGTask,
    DAGTemplate,
    Data,
    DataSource,
    Event,
    EventResponse,
    ExecutorConfig,
    Gauge,
    GCSArtifact,
    GCSArtifactRepository,
    GetUserInfoResponse,
    GitArtifact,
    HDFSArtifact,
    HDFSArtifactRepository,
    Header,
    Histogram,
    HTTPArtifact,
    HTTPAuth,
    HTTPBodySource,
    HTTPHeader as V1alpha1HTTPHeader,
    HTTPHeaderSource,
    InfoResponse,
    Inputs,
    Item,
    LabelKeys,
    LabelValueFrom,
    LabelValues,
    LifecycleHook,
    Link,
    LintCronWorkflowRequest,
    LogEntry as V1alpha1LogEntry,
    ManifestFrom,
    MemoizationStatus,
    Memoize,
    Metadata,
    MetricLabel,
    Metrics,
    Mutex,
    MutexHolding,
    MutexStatus,
    NodeResult,
    NodeStatus,
    NodeSynchronizationStatus,
    NoneStrategy,
    OAuth2Auth,
    OAuth2EndpointParam,
    OSSArtifact,
    OSSArtifactRepository,
    OSSLifecycleRule,
    Outputs,
    ParallelSteps,
    Parameter,
    Plugin,
    PodGC,
    Prometheus,
    RawArtifact,
    ResourceTemplate,
    ResubmitArchivedWorkflowRequest,
    RetryAffinity,
    RetryArchivedWorkflowRequest,
    RetryNodeAntiAffinity,
    RetryStrategy,
    S3Artifact,
    S3ArtifactRepository,
    S3EncryptionOptions,
    ScriptTemplate,
    SemaphoreHolding,
    SemaphoreRef,
    SemaphoreStatus,
    Sequence,
    Submit,
    SubmitOpts,
    SuppliedValueFrom,
    SuspendTemplate,
    Synchronization,
    SynchronizationStatus,
    TarStrategy,
    Template,
    TemplateRef,
    TransformationStep,
    TTLStrategy,
    UpdateCronWorkflowRequest,
    UserContainer,
    ValueFrom,
    Version,
    VolumeClaimGC,
    Workflow,
    WorkflowCreateRequest,
    WorkflowDeleteResponse,
    WorkflowEventBinding,
    WorkflowEventBindingList,
    WorkflowEventBindingSpec,
    WorkflowLintRequest,
    WorkflowList,
    WorkflowMetadata,
    WorkflowResubmitRequest,
    WorkflowResumeRequest,
    WorkflowRetryRequest,
    WorkflowSetRequest,
    WorkflowSpec,
    WorkflowStatus,
    WorkflowStep,
    WorkflowStopRequest,
    WorkflowSubmitRequest,
    WorkflowSuspendRequest,
    WorkflowTaskSetSpec,
    WorkflowTaskSetStatus,
    WorkflowTemplate,
    WorkflowTemplateCreateRequest,
    WorkflowTemplateDeleteResponse,
    WorkflowTemplateLintRequest,
    WorkflowTemplateList,
    WorkflowTemplateRef,
    WorkflowTemplateUpdateRequest,
    WorkflowTerminateRequest,
    WorkflowWatchEvent,
    ZipStrategy,
)
from hera.workflows.models.io.k8s.api.core.v1 import (
    Affinity,
    AWSElasticBlockStoreVolumeSource,
    AzureDiskVolumeSource,
    AzureFileVolumeSource,
    Capabilities,
    CephFSVolumeSource,
    CinderVolumeSource,
    ConfigMapEnvSource,
    ConfigMapKeySelector,
    ConfigMapProjection,
    ConfigMapVolumeSource,
    Container,
    ContainerPort,
    CSIVolumeSource,
    DownwardAPIProjection,
    DownwardAPIVolumeFile,
    DownwardAPIVolumeSource,
    Effect,
    EmptyDirVolumeSource,
    EnvFromSource,
    EnvVar,
    EnvVarSource,
    EphemeralVolumeSource,
    ExecAction,
    FCVolumeSource,
    FlexVolumeSource,
    FlockerVolumeSource,
    GCEPersistentDiskVolumeSource,
    GitRepoVolumeSource,
    GlusterfsVolumeSource,
    GRPCAction,
    HostAlias,
    HostPathVolumeSource,
    HTTPGetAction,
    HTTPHeader as V1HTTPHeader,
    ImagePullPolicy,
    ISCSIVolumeSource,
    KeyToPath,
    Lifecycle,
    LifecycleHandler,
    LocalObjectReference,
    NFSVolumeSource,
    NodeAffinity,
    NodeSelector,
    NodeSelectorRequirement,
    NodeSelectorTerm,
    ObjectFieldSelector,
    ObjectReference,
    Operator,
    OperatorModel,
    PersistentVolumeClaim,
    PersistentVolumeClaimCondition,
    PersistentVolumeClaimSpec,
    PersistentVolumeClaimStatus,
    PersistentVolumeClaimTemplate,
    PersistentVolumeClaimVolumeSource,
    Phase,
    PhotonPersistentDiskVolumeSource,
    PodAffinity,
    PodAffinityTerm,
    PodAntiAffinity,
    PodDNSConfig,
    PodDNSConfigOption,
    PodSecurityContext,
    PortworxVolumeSource,
    PreferredSchedulingTerm,
    Probe,
    ProjectedVolumeSource,
    Protocol,
    QuobyteVolumeSource,
    RBDVolumeSource,
    ResourceFieldSelector,
    ResourceRequirements,
    ScaleIOVolumeSource,
    Scheme,
    SeccompProfile,
    SecretEnvSource,
    SecretKeySelector,
    SecretProjection,
    SecretVolumeSource,
    SecurityContext,
    SELinuxOptions,
    ServiceAccountTokenProjection,
    ServicePort,
    StorageOSVolumeSource,
    Sysctl,
    TCPSocketAction,
    TerminationMessagePolicy,
    Toleration,
    Type,
    TypedLocalObjectReference,
    TypeModel,
    Volume,
    VolumeDevice,
    VolumeMount,
    VolumeProjection,
    VsphereVirtualDiskVolumeSource,
    WeightedPodAffinityTerm,
    WindowsSecurityContextOptions,
)
from hera.workflows.models.io.k8s.api.policy.v1beta1 import PodDisruptionBudgetSpec
from hera.workflows.models.io.k8s.apimachinery.pkg.api.resource import Quantity
from hera.workflows.models.io.k8s.apimachinery.pkg.apis.meta.v1 import (
    CreateOptions,
    FieldsV1,
    GroupVersionResource,
    LabelSelector,
    LabelSelectorRequirement,
    ListMeta,
    ManagedFieldsEntry,
    MicroTime,
    ObjectMeta,
    OwnerReference,
    StatusCause,
    Time,
)
from hera.workflows.models.io.k8s.apimachinery.pkg.util.intstr import IntOrString



"""Internal hera module to help transpile Python to expr.

This module primarily defines user-facing Constant class (aliased as `C` in the public module),
a Parentheses class to parenthesize expressions (aliased as `P` in the public module) and the Identifier
class which provides access to global and closure variables (aliased as `g` and `it` in the public module).
"""

from typing import Any, Dict, List, Optional, Union

# Primitives are the core primitive types available in expr
Primitives = Optional[Union[str, int, bool, float, range]]
Arrays = List[Any]
Maps = Dict[Primitives, Any]
# Constants is a type definition for all acceptable values for the `Constant` class.
Constants = Union[Primitives, Arrays, Maps]


class Node:
    """Node is the base class in our abstract syntax tree which helps Python to expr transpilation.

    It heavily uses the Python data model https://docs.python.org/3/reference/datamodel.html to
    provide a Python native way of constructing expr expressions.

    Once the final expression has been constructed, the python expression can be converted to the expr
    expression by calling str(expression) or repr(expression).
    """

    def __getattr__(self, name: str) -> "GetAttr":
        """Supports attribute access.

        This allows us to provide struct field access. Transpiles `var.<name>` to `var.<name>`.
        """
        return GetAttr(self, name)

    def __getitem__(self, name: str) -> "GetItem":
        """Supports index access.

        This allows us to provide index access. Transpiles `var[<name>]` to `var[<name>]`.
        """
        return GetItem(self, name)

    def __invert__(self):
        """Supports not unary operator.

        Transpiles `~var` to `!var`.
        """
        return UnaryOp(self, "!")

    def __neg__(self):
        """Supports not unary operator.

        Transpiles `-var` to `-var`.
        """
        return UnaryOp(self, "-")

    def __pos__(self):
        """Supports not unary operator.

        Transpiles `+var` to `+var`.
        """
        return UnaryOp(self, "+")

    def __format__(self, format_spec: str) -> str:
        """Supports easy output formatting to construct variable substitution expressions.

        Examples
        --------
        f"{g.input.parameters.value:$}" == "{{input.parameters.value}}"
        f"{g.workflow.parameters.config.jsonpath('$.a'):=}" == "{{=jsonpath(workflow.parameters.config, '$.a')}}"
        f"{g.input.parameters.value}" == "input.parameters.value"
        """
        # For more details around this see https://peps.python.org/pep-3101/
        if not format_spec:
            return repr(self)
        if format_spec == "$":
            return "{{" + repr(self) + "}}"
        if format_spec == "=":
            return "{{=" + repr(self) + "}}"
        raise Exception(f"Invalid format spec '{format_spec}'. Only allowed values are '$' and '='.")

    def length(self):
        """Supports length builtin.

        Transpiles `var.length()` to `len(var)`.
        """
        return Callable("len", self)

    def as_float(self):
        """Supports asFloat builtin.

        Transpiles `var.as_float()` to `asFloat(var)`.
        """
        return Callable("asFloat", self)

    def as_int(self):
        """Supports asInt builtin.

        Transpiles `var.as_int()` to `asInt(var)`.
        """
        return Callable("asInt", self)

    def check(self, truthy_value: "Node", falsy_value: "Node") -> "Check":
        """Supports ternary operator.

        Transpiles `var.check(truthy_value, falsy_value)` to `var ? truthy_value : falsy_value`.
        """
        return Check(self, truthy_value, falsy_value)

    def get(self, name: str) -> "GetAttr":
        """Supports attribute access.

        This allows us to provide struct field access in case the field name conflicts with one of the Node methods.
        Transpiles `var.get(<name>)` to `var.<name>`.
        """
        return GetAttr(self, name)

    def jsonpath(self, path: str):
        """Supports jsonpath builtin.

        Transpiles `var.jsonpath(<path>)` to `jsonpath(var, <path>)`.
        """
        return Callable("jsonpath", self, Constant(str(path)))

    def string(self):
        """Supports string builtin.

        Transpiles `var.string()` to `string(var)`.
        """
        return Callable("string", self)

    def to_json(self):
        """Supports toJson builtin.

        Transpiles `var.to_json()` to `toJson(var)`.
        """
        return Callable("toJson", self)


# A list of binary operations supported by each node.
# Mapping what is available at
# https://github.com/antonmedv/expr/blob/34d2cbe7e43af1348d7010d951f1d7fe44a91500/checker/checker.go#L198
BINARY_OP_MAP = {
    # Refer to the Python data model to understand the dunder methods.
    # https://docs.python.org/3/reference/datamodel.html#emulating-numeric-types
    # https://docs.python.org/3/reference/datamodel.html#object.__lt__
    # For all of these dunder methods, we transpile python statements like <node1> <python-operator> <node2>
    # to <node1> <expr-operator> <node2> statements. The dunder method names defined as keys
    # specify the way users can express the operation in python and the values
    # specify the operator in expr it will be transpiled to.
    "__add__": "+",
    "__and__": "&&",
    "__eq__": "==",
    "__ge__": ">=",
    "__gt__": ">",
    "__le__": "<=",
    "__lt__": "<",
    "__mod__": "%",
    "__mul__": "*",
    "__ne__": "!=",
    "__or__": "||",
    "__pow__": "**",
    "__sub__": "-",
    "__truediv__": "/",
    # These methods translate <node1>.<python-method>(<node2>) to <node1> <expr-operator> <node2>
    "contains": "contains",
    "ends_with": "endsWith",
    "in_": "in",
    "matches": "matches",
    "not_in": "not in",
    "starts_with": "startsWith",
}

# This defines a list of builtins that support a predicate along with a local
# iterator. See https://github.com/antonmedv/expr/blob/master/docs/Language-Definition.md#builtin-functions
# and https://github.com/antonmedv/expr/blob/master/docs/Language-Definition.md#closures for more details.
BUILTINS = (
    "all",
    "any",
    "count",
    "filter",
    "map",
    "none",
    "one",
)

# Here we dynamically set all the binary operators on the Node class
for method, op in BINARY_OP_MAP.items():
    # Note, we need a nested function here to properly bind
    # the op variable.
    # See https://eev.ee/blog/2011/04/24/gotcha-python-scoping-closures/#the-solution
    def operator(op: str):
        def func(self: Node, other: Node) -> "BinaryOp":
            if not isinstance(other, Node):
                other = Constant(other)
            return BinaryOp(self, other, op)

        return func

    setattr(Node, method, operator(op))

# Here we dynamically set all the builtin operators on the Node class
for builtin in BUILTINS:
    # Note, we need a nested function here to properly bind
    # the builtin variable.
    # See https://eev.ee/blog/2011/04/24/gotcha-python-scoping-closures/#the-solution
    def _builtin_func(builtin):
        def func(self: Node, operation: Node) -> "Builtin":
            return Builtin(builtin, self, operation)

        return func

    setattr(Node, builtin, _builtin_func(builtin))


def _constant_repr(obj):
    if obj is None:
        return "nil"
    if obj is True:
        return "true"
    if obj is False:
        return "false"
    if isinstance(obj, range):
        return f"{obj.start}..{obj.stop - 1}"
    if isinstance(obj, list):
        return f'[{", ".join(map(_constant_repr, obj))}]'
    if isinstance(obj, dict):
        key_value_pairs = [f"{_constant_repr(key)}: {_constant_repr(value)}" for key, value in obj.items()]
        return f'{{{", ".join(key_value_pairs)}}}'
    return repr(obj)


class Constant(Node):
    """Supports transpiling inline python constants to expr expressions.

    Examples
    --------
    str(C(1)) == 1
    str(C(True)) == true
    str(C(None)) == nil
    str(C([1, 2, 3])) == [1, 2, 3]
    """

    def __init__(self, value: Constants):
        if isinstance(value, range):
            if value.step != 1:
                raise Exception("Only ranges with a step size of 1 are allowed")
        self.value = value

    def __repr__(self) -> str:
        # we need a custom repr function in order to recursively translate
        # True -> true, False -> false and None -> nil
        return _constant_repr(self.value)


class Identifier(Node):
    """Internal class used to define the `g` and `it` context variables."""

    def __init__(self, value: str = ""):
        self.value = value

    def __repr__(self) -> str:
        return self.value


class Parentheses(Node):
    """Supports transpiling groups of Python expressions so that they are parathesized properly in expr.

    Examples
    --------
    str(P(C(1) + C(2)) + 3) == (1 + 2) + 3
    """

    def __init__(self, value: Node):
        self.value = value

    def __repr__(self) -> str:
        return f"({self.value})"


class BinaryOp(Node):
    """Supports transpiling binary operators. See Node class for more information."""

    def __init__(self, value: Node, other_value: Node, operation: str):
        self.value = value
        self.other_value = other_value
        self.operation = operation

    def __repr__(self) -> str:
        return f"{self.value} {self.operation} {self.other_value}"


class UnaryOp(Node):
    """Supports transpiling unary operators. See Node class for more information."""

    def __init__(self, value: Node, operation: str):
        self.value = value
        self.operation = operation

    def __repr__(self) -> str:
        return f"{self.operation}{self.value}"


class Callable(Node):
    """Supports transpiling callables. See Node class for more information."""

    def __init__(self, function: str, *args: Any):
        self.function = function
        self.args = ", ".join(map(repr, map(lambda node: node if isinstance(node, Node) else Constant(node), args)))

    def __repr__(self) -> str:
        return f"{self.function}({self.args})"


class GetAttr(Node):
    """Supports transpiling attribute access. See Node class for more information."""

    def __init__(self, value: Node, attribute: str):
        self.value = value
        self.attribute = attribute

    def __repr__(self) -> str:
        return f"{self.value}.{self.attribute}" if str(self.value) else str(self.attribute)


class GetItem(Node):
    """Supports transpiling index access. See Node class for more information."""

    def __init__(self, value: Node, attribute: Union[str, int]):
        self.value = value
        if isinstance(attribute, slice):
            if attribute.step and attribute.step != 1:
                raise Exception("Only slices with a step size of 1 are allowed")
            start = attribute.start if attribute.start is not None else ""
            stop = attribute.stop if attribute.stop is not None else ""
            self.attribute = f"{start}:{stop}"
        else:
            self.attribute = repr(attribute)

    def __repr__(self) -> str:
        return f"{self.value}[{self.attribute}]"


class Builtin(Node):
    """Supports transpiling builtin expressions with predicates. See Node class for more information."""

    def __init__(self, operator: str, operand: Node, operation: Any):
        self.operator = operator
        self.operand = operand
        self.operation = operation if isinstance(operation, Node) else Constant(operation)

    def __repr__(self) -> str:
        return f"{self.operator}({self.operand}, {{{self.operation}}})"


class Check(Node):
    """Supports transpiling ternary operator. See Node class for more information."""

    def __init__(self, value: Node, truthy_value: Node, falsy_value: Node):
        self.value = value
        self.truthy_value = truthy_value
        self.falsy_value = falsy_value

    def __repr__(self) -> str:
        return f"{self.value} ? {self.truthy_value} : {self.falsy_value}"



"""The suspend module provides the Suspend class. The Suspend template in Hera
provides a convenience wrapper around the "Intermediate Parameters" Argo feature.

See https://argoproj.github.io/argo-workflows/walk-through/suspending/
for more on suspending.

See https://argoproj.github.io/argo-workflows/intermediate-inputs/
for more on intermediate parameters.

"""
from typing import List, Optional, Union

from hera.workflows._mixins import TemplateMixin
from hera.workflows.models import (
    Inputs,
    Outputs,
    SuspendTemplate as _ModelSuspendTemplate,
    Template as _ModelTemplate,
)
from hera.workflows.parameter import Parameter


class Suspend(TemplateMixin):
    """The Suspend template allows the user to pause a workflow for a specified length of time
    given by `duration` or indefinitely (i.e. until manually resumed). The Suspend template also
    allows you to specify `intermediate_parameters` which will replicate the given parameters to
    the "inputs" and "outputs" of the template, resulting in a Suspend template that pauses and
    waits for values from the user for the given list of parameters.
    """

    duration: Optional[Union[int, str]] = None
    intermediate_parameters: List[Parameter] = []

    def _build_suspend_template(self) -> _ModelSuspendTemplate:
        return _ModelSuspendTemplate(
            duration=self.duration,
        )

    def _build_outputs(self) -> Optional[Outputs]:
        outputs = []
        for param in self.intermediate_parameters:
            outputs.append(
                Parameter(name=param.name, value_from={"supplied": {}}, description=param.description).as_output()
            )
        return Outputs(parameters=outputs) if outputs else None

    def _build_inputs(self) -> Optional[Inputs]:
        inputs = []
        for param in self.intermediate_parameters:
            inputs.append(param.as_input())
        return Inputs(parameters=inputs) if inputs else None

    def _build_template(self) -> _ModelTemplate:
        return _ModelTemplate(
            active_deadline_seconds=self.active_deadline_seconds,
            affinity=self.affinity,
            archive_location=self.archive_location,
            automount_service_account_token=self.automount_service_account_token,
            executor=self.executor,
            fail_fast=self.fail_fast,
            host_aliases=self.host_aliases,
            init_containers=self.init_containers,
            inputs=self._build_inputs(),
            memoize=self.memoize,
            metadata=self._build_metadata(),
            name=self.name,
            node_selector=self.node_selector,
            outputs=self._build_outputs(),
            plugin=self.plugin,
            priority_class_name=self.priority_class_name,
            priority=self.priority,
            retry_strategy=self.retry_strategy,
            scheduler_name=self.scheduler_name,
            security_context=self.pod_security_context,
            service_account_name=self.service_account_name,
            sidecars=self.sidecars,
            suspend=self._build_suspend_template(),
            synchronization=self.synchronization,
            timeout=self.timeout,
            tolerations=self.tolerations,
        )


__all__ = ["Suspend"]



"""The script module provides the Script class.

See https://argoproj.github.io/argo-workflows/workflow-concepts/#script
for more on scripts.
"""
import copy
import inspect
import textwrap
from typing import Callable, Dict, List, Optional, Union

from hera.shared import global_config
from hera.workflows._mixins import (
    CallableTemplateMixin,
    ContainerMixin,
    EnvIOMixin,
    ResourceMixin,
    TemplateMixin,
    VolumeMountMixin,
)
from hera.workflows.models import (
    Inputs as ModelInputs,
    Lifecycle,
    ScriptTemplate as _ModelScriptTemplate,
    SecurityContext,
    Template as _ModelTemplate,
)
from hera.workflows.parameter import MISSING, Parameter
from hera.workflows.steps import Step
from hera.workflows.task import Task


class Script(
    EnvIOMixin,
    CallableTemplateMixin,
    ContainerMixin,
    TemplateMixin,
    ResourceMixin,
    VolumeMountMixin,
):
    """A Script acts as a wrapper around a container. In Hera this defaults to a "python:3.7" image
    specified by global_config.image, which runs a python source specified by `Script.source`.
    """

    container_name: Optional[str] = None
    args: Optional[List[str]] = None
    command: Optional[List[str]] = global_config.script_command
    lifecycle: Optional[Lifecycle] = None
    security_context: Optional[SecurityContext] = None
    source: Optional[Union[Callable, str]] = None
    working_dir: Optional[str] = None
    add_cwd_to_sys_path: bool = True

    def _get_param_script_portion(self) -> str:
        """Constructs and returns a script that loads the parameters of the specified arguments. Since Argo passes
        parameters through {{input.parameters.name}} it can be very cumbersome for users to manage that. This creates a
        script that automatically imports json and loads/adds code to interpret each independent argument into the
        script.

        Returns
        -------
        str
            The string representation of the script to load.
        """
        inputs = self._build_inputs()
        assert inputs
        extract = "import json\n"
        for param in sorted(inputs.parameters or [], key=lambda x: x.name):
            # Hera does not know what the content of the `InputFrom` is, coming from another task. In some cases
            # non-JSON encoded strings are returned, which fail the loads, but they can be used as plain strings
            # which is why this captures that in an except. This is only used for `InputFrom` cases as the extra
            # payload of the script is not necessary when regular input is set on the task via `func_params`
            extract += f"""try: {param.name} = json.loads(r'''{{{{inputs.parameters.{param.name}}}}}''')\n"""
            extract += f"""except: {param.name} = r'''{{{{inputs.parameters.{param.name}}}}}'''\n"""
        return textwrap.dedent(extract)

    def _build_source(self) -> str:
        """Assembles and returns a script representation of the given function, along with the extra script material
        prefixed to the string. The script is expected to be a callable function the client is interested in submitting
        for execution on Argo and the script_extra material represents the parameter loading part obtained, likely,
        through get_param_script_portion.

        Returns
        -------
        str
            Final formatted script.
        """
        if callable(self.source):
            signature = inspect.signature(self.source)
            args = inspect.getfullargspec(self.source).args
            if signature.return_annotation == str:
                # Resolve function by filling in templated inputs
                input_params_names = [p.name for p in self.inputs if isinstance(p, Parameter)]  # type: ignore
                missing_args = set(args) - set(input_params_names)
                if missing_args:
                    raise ValueError(f"Missing inputs for source args: {missing_args}")
                kwargs = {name: f"{{{{inputs.parameters.{name}}}}}" for name in args}
                # Resolve the function to a string
                return self.source(**kwargs)
            else:
                script = ""
                # Argo will save the script as a file and run it with cmd:
                # - python /argo/staging/script
                # However, this prevents the script from importing modules in its cwd,
                # since it's looking for files relative to the script path.
                # We fix this by appending the cwd path to sys:
                if self.add_cwd_to_sys_path:
                    script = "import os\nimport sys\nsys.path.append(os.getcwd())\n"

                script_extra = self._get_param_script_portion() if args else None
                if script_extra:
                    script += copy.deepcopy(script_extra)
                    script += "\n"

                # content represents the function components, separated by new lines
                # therefore, the actual code block occurs after the end parenthesis, which is a literal `):\n`
                content = inspect.getsourcelines(self.source)[0]
                token_index, start_token = 1, ":\n"
                for curr_index, curr_token in enumerate(content):
                    if start_token in curr_token:
                        # when we find the curr token we find the end of the function header. The next index is the
                        # starting point of the function body
                        token_index = curr_index + 1
                        break

                s = "".join(content[token_index:])
                script += textwrap.dedent(s)
                return textwrap.dedent(script)
        else:
            assert isinstance(self.source, str)
            return self.source

    def _build_inputs(self) -> Optional[ModelInputs]:
        inputs = super()._build_inputs()
        func_parameters = _get_parameters_from_callable(self.source) if callable(self.source) else None

        if inputs is None and func_parameters is None:
            return None
        elif func_parameters is None:
            return inputs
        elif inputs is None:
            inputs = ModelInputs(parameters=func_parameters)

        already_set_params = {p.name for p in inputs.parameters or []}
        for param in func_parameters:
            if param.name not in already_set_params:
                inputs.parameters = [param] if inputs.parameters is None else inputs.parameters + [param]
        return inputs

    def _build_template(self) -> _ModelTemplate:
        return _ModelTemplate(
            active_deadline_seconds=self.active_deadline_seconds,
            affinity=self.affinity,
            archive_location=self.archive_location,
            automount_service_account_token=self.automount_service_account_token,
            daemon=self.daemon,
            executor=self.executor,
            fail_fast=self.fail_fast,
            host_aliases=self.host_aliases,
            init_containers=self.init_containers,
            inputs=self._build_inputs(),
            memoize=self.memoize,
            metadata=self._build_metadata(),
            metrics=self.metrics,
            name=self.name,
            node_selector=self.node_selector,
            outputs=self._build_outputs(),
            parallelism=self.parallelism,
            plugin=self.plugin,
            pod_spec_patch=self.pod_spec_patch,
            priority=self.priority,
            priority_class_name=self.priority_class_name,
            resource=self._build_resources(),
            retry_strategy=self.retry_strategy,
            scheduler_name=self.scheduler_name,
            script=self._build_script(),
            security_context=self.pod_security_context,
            service_account_name=self.service_account_name,
            sidecars=self.sidecars,
            synchronization=self.synchronization,
            timeout=self.timeout,
            tolerations=self.tolerations,
            volumes=self._build_volumes(),
        )

    def _build_script(self) -> _ModelScriptTemplate:
        return _ModelScriptTemplate(
            args=self.args,
            command=self.command,
            env=self._build_env(),
            env_from=self._build_env_from(),
            image=self.image,
            image_pull_policy=self._build_image_pull_policy(),
            lifecycle=self.lifecycle,
            liveness_probe=self.liveness_probe,
            name=self.container_name,
            ports=self.ports,
            readiness_probe=self.readiness_probe,
            resources=self._build_resources(),
            security_context=self.security_context,
            source=self._build_source(),
            startup_probe=self.startup_probe,
            stdin=self.stdin,
            stdin_once=self.stdin_once,
            termination_message_path=self.termination_message_path,
            termination_message_policy=self.termination_message_policy,
            tty=self.tty,
            volume_devices=self.volume_devices,
            volume_mounts=self._build_volume_mounts(),
            working_dir=self.working_dir,
        )


def _get_parameters_from_callable(source: Callable) -> Optional[List[Parameter]]:
    # If there are any kwargs arguments associated with the function signature,
    # we store these as we can set them as default values for argo arguments
    source_signature: Dict[str, Optional[object]] = {}
    for p in inspect.signature(source).parameters.values():
        if p.default != inspect.Parameter.empty and p.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD:
            source_signature[p.name] = p.default
        else:
            source_signature[p.name] = MISSING

    if len(source_signature) == 0:
        return None
    return [Parameter(name=n, default=v) for n, v in source_signature.items()]


def script(**script_kwargs):
    """A decorator that wraps a function into a Script object.

    Using this decorator users can define a function that will be executed as a script in a container. Once the
    `Script` is returned users can use it as they generally use a `Script` e.g. as a callable inside a DAG or Steps.
    Note that invoking the function will result in the template associated with the script to be added to the
    workflow context, so users do not have to worry about that.

    Parameters
    ----------
    **script_kwargs
        Keyword arguments to be passed to the Script object.

    Returns
    -------
    Callable
        Function that wraps a given function into a `Script`.
    """

    def script_wrapper(func: Callable) -> Callable:
        """Wraps the given callable into a `Script` object that can be invoked.

        Parameters
        ----------
        func: Callable
            Function to wrap.

        Returns
        -------
        Callable
            Another callable that represents the `Script` object `__call__` method.
        """
        s = Script(name=func.__name__.replace("_", "-"), source=func, **script_kwargs)

        def task_wrapper(**task_params) -> Union[Task, Step]:
            """Invokes a `Script` object's `__call__` method using the given `task_params`"""
            return s.__call__(**task_params)  # type: ignore

        return task_wrapper

    return script_wrapper


__all__ = ["Script", "script"]



# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/argoproj/argo-workflows/v3.4.4/api/openapi-spec/swagger.json

from __future__ import annotations

from datetime import datetime
from typing import Dict, List, Optional

from pydantic import Field

from hera.shared._base_model import BaseModel


class CreateOptions(BaseModel):
    dry_run: Optional[List[str]] = Field(
        None,
        alias="dryRun",
        title=(
            "When present, indicates that modifications should not be\npersisted. An invalid or unrecognized dryRun"
            " directive will\nresult in an error response and no further processing of the\nrequest. Valid values"
            " are:\n- All: all dry run stages will be processed\n+optional"
        ),
    )
    field_manager: Optional[str] = Field(
        None,
        alias="fieldManager",
        title=(
            "fieldManager is a name associated with the actor or entity\nthat is making these changes. The value must"
            " be less than or\n128 characters long, and only contain printable characters,\nas defined by"
            " https://golang.org/pkg/unicode/#IsPrint.\n+optional"
        ),
    )
    field_validation: Optional[str] = Field(
        None,
        alias="fieldValidation",
        title=(
            "fieldValidation instructs the server on how to handle\nobjects in the request (POST/PUT/PATCH) containing"
            " unknown\nor duplicate fields, provided that the `ServerSideFieldValidation`\nfeature gate is also"
            " enabled. Valid values are:\n- Ignore: This will ignore any unknown fields that are silently\ndropped"
            " from the object, and will ignore all but the last duplicate\nfield that the decoder encounters. This is"
            " the default behavior\nprior to v1.23 and is the default behavior when the\n`ServerSideFieldValidation`"
            " feature gate is disabled.\n- Warn: This will send a warning via the standard warning response\nheader"
            " for each unknown field that is dropped from the object, and\nfor each duplicate field that is"
            " encountered. The request will\nstill succeed if there are no other errors, and will only persist\nthe"
            " last of any duplicate fields. This is the default when the\n`ServerSideFieldValidation` feature gate is"
            " enabled.\n- Strict: This will fail the request with a BadRequest error if\nany unknown fields would be"
            " dropped from the object, or if any\nduplicate fields are present. The error returned from the"
            " server\nwill contain all unknown and duplicate fields encountered.\n+optional"
        ),
    )


class FieldsV1(BaseModel):
    pass


class GroupVersionResource(BaseModel):
    group: Optional[str] = None
    resource: Optional[str] = None
    version: Optional[str] = None


class LabelSelectorRequirement(BaseModel):
    key: str = Field(..., description="key is the label key that the selector applies to.")
    operator: str = Field(
        ...,
        description=(
            "operator represents a key's relationship to a set of values. Valid operators are In, NotIn, Exists and"
            " DoesNotExist."
        ),
    )
    values: Optional[List[str]] = Field(
        None,
        description=(
            "values is an array of string values. If the operator is In or NotIn, the values array must be non-empty."
            " If the operator is Exists or DoesNotExist, the values array must be empty. This array is replaced during"
            " a strategic merge patch."
        ),
    )


class ListMeta(BaseModel):
    continue_: Optional[str] = Field(
        None,
        alias="continue",
        description=(
            "continue may be set if the user set a limit on the number of items returned, and indicates that the"
            " server has more data available. The value is opaque and may be used to issue another request to the"
            " endpoint that served this list to retrieve the next set of available objects. Continuing a consistent"
            " list may not be possible if the server configuration has changed or more than a few minutes have passed."
            " The resourceVersion field returned when using this continue value will be identical to the value in the"
            " first response, unless you have received this token from an error message."
        ),
    )
    remaining_item_count: Optional[int] = Field(
        None,
        alias="remainingItemCount",
        description=(
            "remainingItemCount is the number of subsequent items in the list which are not included in this list"
            " response. If the list request contained label or field selectors, then the number of remaining items is"
            " unknown and the field will be left unset and omitted during serialization. If the list is complete"
            " (either because it is not chunking or because this is the last chunk), then there are no more remaining"
            " items and this field will be left unset and omitted during serialization. Servers older than v1.15 do"
            " not set this field. The intended use of the remainingItemCount is *estimating* the size of a collection."
            " Clients should not rely on the remainingItemCount to be set or to be exact."
        ),
    )
    resource_version: Optional[str] = Field(
        None,
        alias="resourceVersion",
        description=(
            "String that identifies the server's internal version of this object that can be used by clients to"
            " determine when objects have changed. Value must be treated as opaque by clients and passed unmodified"
            " back to the server. Populated by the system. Read-only. More info:"
            " https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency"
        ),
    )
    self_link: Optional[str] = Field(
        None,
        alias="selfLink",
        description=(
            "selfLink is a URL representing this object. Populated by the system. Read-only.\n\nDEPRECATED Kubernetes"
            " will stop propagating this field in 1.20 release and the field is planned to be removed in 1.21 release."
        ),
    )


class MicroTime(BaseModel):
    __root__: datetime = Field(..., description="MicroTime is version of Time with microsecond level precision.")


class OwnerReference(BaseModel):
    api_version: str = Field(..., alias="apiVersion", description="API version of the referent.")
    block_owner_deletion: Optional[bool] = Field(
        None,
        alias="blockOwnerDeletion",
        description=(
            'If true, AND if the owner has the "foregroundDeletion" finalizer, then the owner cannot be deleted from'
            " the key-value store until this reference is removed. Defaults to false. To set this field, a user needs"
            ' "delete" permission of the owner, otherwise 422 (Unprocessable Entity) will be returned.'
        ),
    )
    controller: Optional[bool] = Field(None, description="If true, this reference points to the managing controller.")
    kind: str = Field(
        ...,
        description=(
            "Kind of the referent. More info:"
            " https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    )
    name: str = Field(
        ..., description="Name of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#names"
    )
    uid: str = Field(
        ..., description="UID of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#uids"
    )


class StatusCause(BaseModel):
    field: Optional[str] = Field(
        None,
        description=(
            "The field of the resource that has caused this error, as named by its JSON serialization. May include dot"
            " and postfix notation for nested attributes. Arrays are zero-indexed.  Fields may appear more than once"
            ' in an array of causes due to fields having multiple errors. Optional.\n\nExamples:\n  "name" - the field'
            ' "name" on the current resource\n  "items[0].name" - the field "name" on the first array entry in "items"'
        ),
    )
    message: Optional[str] = Field(
        None,
        description=(
            "A human-readable description of the cause of the error.  This field may be presented as-is to a reader."
        ),
    )
    reason: Optional[str] = Field(
        None,
        description=(
            "A machine-readable description of the cause of the error. If this value is empty there is no information"
            " available."
        ),
    )


class Time(BaseModel):
    __root__: datetime = Field(
        ...,
        description=(
            "Time is a wrapper around time.Time which supports correct marshaling to YAML and JSON.  Wrappers are"
            " provided for many of the factory methods that the time package offers."
        ),
    )


class LabelSelector(BaseModel):
    match_expressions: Optional[List[LabelSelectorRequirement]] = Field(
        None,
        alias="matchExpressions",
        description="matchExpressions is a list of label selector requirements. The requirements are ANDed.",
    )
    match_labels: Optional[Dict[str, str]] = Field(
        None,
        alias="matchLabels",
        description=(
            "matchLabels is a map of {key,value} pairs. A single {key,value} in the matchLabels map is equivalent to"
            ' an element of matchExpressions, whose key field is "key", the operator is "In", and the values array'
            ' contains only "value". The requirements are ANDed.'
        ),
    )


class ManagedFieldsEntry(BaseModel):
    api_version: Optional[str] = Field(
        None,
        alias="apiVersion",
        description=(
            "APIVersion defines the version of this resource that this field set applies to. The format is"
            ' "group/version" just like the top-level APIVersion field. It is necessary to track the version of a'
            " field set because it cannot be automatically converted."
        ),
    )
    fields_type: Optional[str] = Field(
        None,
        alias="fieldsType",
        description=(
            "FieldsType is the discriminator for the different fields format and version. There is currently only one"
            ' possible value: "FieldsV1"'
        ),
    )
    fields_v1: Optional[FieldsV1] = Field(
        None,
        alias="fieldsV1",
        description='FieldsV1 holds the first JSON version format as described in the "FieldsV1" type.',
    )
    manager: Optional[str] = Field(None, description="Manager is an identifier of the workflow managing these fields.")
    operation: Optional[str] = Field(
        None,
        description=(
            "Operation is the type of operation which lead to this ManagedFieldsEntry being created. The only valid"
            " values for this field are 'Apply' and 'Update'."
        ),
    )
    subresource: Optional[str] = Field(
        None,
        description=(
            "Subresource is the name of the subresource used to update that object, or empty string if the object was"
            " updated through the main resource. The value of this field is used to distinguish between managers, even"
            " if they share the same name. For example, a status update will be distinct from a regular update using"
            " the same manager name. Note that the APIVersion field is not related to the Subresource field and it"
            " always corresponds to the version of the main resource."
        ),
    )
    time: Optional[Time] = Field(
        None,
        description=(
            "Time is timestamp of when these fields were set. It should always be empty if Operation is 'Apply'"
        ),
    )


class ObjectMeta(BaseModel):
    annotations: Optional[Dict[str, str]] = Field(
        None,
        description=(
            "Annotations is an unstructured key value map stored with a resource that may be set by external tools to"
            " store and retrieve arbitrary metadata. They are not queryable and should be preserved when modifying"
            " objects. More info: http://kubernetes.io/docs/user-guide/annotations"
        ),
    )
    cluster_name: Optional[str] = Field(
        None,
        alias="clusterName",
        description=(
            "The name of the cluster which the object belongs to. This is used to distinguish resources with same name"
            " and namespace in different clusters. This field is not set anywhere right now and apiserver is going to"
            " ignore it if set in create or update request."
        ),
    )
    creation_timestamp: Optional[Time] = Field(
        None,
        alias="creationTimestamp",
        description=(
            "CreationTimestamp is a timestamp representing the server time when this object was created. It is not"
            " guaranteed to be set in happens-before order across separate operations. Clients may not set this value."
            " It is represented in RFC3339 form and is in UTC.\n\nPopulated by the system. Read-only. Null for lists."
            " More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    )
    deletion_grace_period_seconds: Optional[int] = Field(
        None,
        alias="deletionGracePeriodSeconds",
        description=(
            "Number of seconds allowed for this object to gracefully terminate before it will be removed from the"
            " system. Only set when deletionTimestamp is also set. May only be shortened. Read-only."
        ),
    )
    deletion_timestamp: Optional[Time] = Field(
        None,
        alias="deletionTimestamp",
        description=(
            "DeletionTimestamp is RFC 3339 date and time at which this resource will be deleted. This field is set by"
            " the server when a graceful deletion is requested by the user, and is not directly settable by a client."
            " The resource is expected to be deleted (no longer visible from resource lists, and not reachable by"
            " name) after the time in this field, once the finalizers list is empty. As long as the finalizers list"
            " contains items, deletion is blocked. Once the deletionTimestamp is set, this value may not be unset or"
            " be set further into the future, although it may be shortened or the resource may be deleted prior to"
            " this time. For example, a user may request that a pod is deleted in 30 seconds. The Kubelet will react"
            " by sending a graceful termination signal to the containers in the pod. After that 30 seconds, the"
            " Kubelet will send a hard termination signal (SIGKILL) to the container and after cleanup, remove the pod"
            " from the API. In the presence of network partitions, this object may still exist after this timestamp,"
            " until an administrator or automated process can determine the resource is fully terminated. If not set,"
            " graceful deletion of the object has not been requested.\n\nPopulated by the system when a graceful"
            " deletion is requested. Read-only. More info:"
            " https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    )
    finalizers: Optional[List[str]] = Field(
        None,
        description=(
            "Must be empty before the object is deleted from the registry. Each entry is an identifier for the"
            " responsible component that will remove the entry from the list. If the deletionTimestamp of the object"
            " is non-nil, entries in this list can only be removed. Finalizers may be processed and removed in any"
            " order.  Order is NOT enforced because it introduces significant risk of stuck finalizers. finalizers is"
            " a shared field, any actor with permission can reorder it. If the finalizer list is processed in order,"
            " then this can lead to a situation in which the component responsible for the first finalizer in the list"
            " is waiting for a signal (field value, external system, or other) produced by a component responsible for"
            " a finalizer later in the list, resulting in a deadlock. Without enforced ordering finalizers are free to"
            " order amongst themselves and are not vulnerable to ordering changes in the list."
        ),
    )
    generate_name: Optional[str] = Field(
        None,
        alias="generateName",
        description=(
            "GenerateName is an optional prefix, used by the server, to generate a unique name ONLY IF the Name field"
            " has not been provided. If this field is used, the name returned to the client will be different than the"
            " name passed. This value will also be combined with a unique suffix. The provided value has the same"
            " validation rules as the Name field, and may be truncated by the length of the suffix required to make"
            " the value unique on the server.\n\nIf this field is specified and the generated name exists, the server"
            " will NOT return a 409 - instead, it will either return 201 Created or 500 with Reason ServerTimeout"
            " indicating a unique name could not be found in the time allotted, and the client should retry"
            " (optionally after the time indicated in the Retry-After header).\n\nApplied only if Name is not"
            " specified. More info:"
            " https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#idempotency"
        ),
    )
    generation: Optional[int] = Field(
        None,
        description=(
            "A sequence number representing a specific generation of the desired state. Populated by the system."
            " Read-only."
        ),
    )
    labels: Optional[Dict[str, str]] = Field(
        None,
        description=(
            "Map of string keys and values that can be used to organize and categorize (scope and select) objects. May"
            " match selectors of replication controllers and services. More info:"
            " http://kubernetes.io/docs/user-guide/labels"
        ),
    )
    managed_fields: Optional[List[ManagedFieldsEntry]] = Field(
        None,
        alias="managedFields",
        description=(
            "ManagedFields maps workflow-id and version to the set of fields that are managed by that workflow. This"
            " is mostly for internal housekeeping, and users typically shouldn't need to set or understand this"
            " field. A workflow can be the user's name, a controller's name, or the name of a specific apply path"
            ' like "ci-cd". The set of fields is always in the version that the workflow used when modifying the'
            " object."
        ),
    )
    name: Optional[str] = Field(
        None,
        description=(
            "Name must be unique within a namespace. Is required when creating resources, although some resources may"
            " allow a client to request the generation of an appropriate name automatically. Name is primarily"
            " intended for creation idempotence and configuration definition. Cannot be updated. More info:"
            " http://kubernetes.io/docs/user-guide/identifiers#names"
        ),
    )
    namespace: Optional[str] = Field(
        None,
        description=(
            "Namespace defines the space within which each name must be unique. An empty namespace is equivalent to"
            ' the "default" namespace, but "default" is the canonical representation. Not all objects are required to'
            " be scoped to a namespace - the value of this field for those objects will be empty.\n\nMust be a"
            " DNS_LABEL. Cannot be updated. More info: http://kubernetes.io/docs/user-guide/namespaces"
        ),
    )
    owner_references: Optional[List[OwnerReference]] = Field(
        None,
        alias="ownerReferences",
        description=(
            "List of objects depended by this object. If ALL objects in the list have been deleted, this object will"
            " be garbage collected. If this object is managed by a controller, then an entry in this list will point"
            " to this controller, with the controller field set to true. There cannot be more than one managing"
            " controller."
        ),
    )
    resource_version: Optional[str] = Field(
        None,
        alias="resourceVersion",
        description=(
            "An opaque value that represents the internal version of this object that can be used by clients to"
            " determine when objects have changed. May be used for optimistic concurrency, change detection, and the"
            " watch operation on a resource or set of resources. Clients must treat these values as opaque and passed"
            " unmodified back to the server. They may only be valid for a particular resource or set of"
            " resources.\n\nPopulated by the system. Read-only. Value must be treated as opaque by clients and . More"
            " info:"
            " https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency"
        ),
    )
    self_link: Optional[str] = Field(
        None,
        alias="selfLink",
        description=(
            "SelfLink is a URL representing this object. Populated by the system. Read-only.\n\nDEPRECATED Kubernetes"
            " will stop propagating this field in 1.20 release and the field is planned to be removed in 1.21 release."
        ),
    )
    uid: Optional[str] = Field(
        None,
        description=(
            "UID is the unique in time and space value for this object. It is typically generated by the server on"
            " successful creation of a resource and is not allowed to change on PUT operations.\n\nPopulated by the"
            " system. Read-only. More info: http://kubernetes.io/docs/user-guide/identifiers#uids"
        ),
    )



# generated by datamodel-codegen:
#   filename:  https://raw.githubusercontent.com/argoproj/argo-workflows/v3.4.4/api/openapi-spec/swagger.json

from __future__ import annotations

from typing import Optional

from pydantic import Field

from hera.shared._base_model import BaseModel

from .io.argoproj.events import v1alpha1
from .io.k8s.apimachinery.pkg.apis.meta import v1


class EventSourceDeletedResponse(BaseModel):
    pass


class LogEntry(BaseModel):
    event_name: Optional[str] = Field(None, alias="eventName", title="optional - the event name (e.g. `example`)")
    event_source_name: Optional[str] = Field(None, alias="eventSourceName")
    event_source_type: Optional[str] = Field(
        None, alias="eventSourceType", title="optional - the event source type (e.g. `webhook`)"
    )
    level: Optional[str] = None
    msg: Optional[str] = None
    namespace: Optional[str] = None
    time: Optional[v1.Time] = None


class CreateEventSourceRequest(BaseModel):
    event_source: Optional[v1alpha1.EventSource] = Field(None, alias="eventSource")
    namespace: Optional[str] = None


class EventSourceWatchEvent(BaseModel):
    object: Optional[v1alpha1.EventSource] = None
    type: Optional[str] = None


class UpdateEventSourceRequest(BaseModel):
    event_source: Optional[v1alpha1.EventSource] = Field(None, alias="eventSource")
    name: Optional[str] = None
    namespace: Optional[str] = None



from typing import List, Optional

from hera.shared._base_model import BaseModel as BaseModel

from ...google import protobuf as protobuf

class Error(BaseModel):
    code: Optional[int]
    details: Optional[List[protobuf.Any]]
    error: Optional[str]
    message: Optional[str]

class StreamError(BaseModel):
    details: Optional[List[protobuf.Any]]
    grpc_code: Optional[int]
    http_code: Optional[int]
    http_status: Optional[str]
    message: Optional[str]



class InvalidType(Exception):
    ...


class InvalidTemplateCall(Exception):
    ...


class InvalidDispatchType(Exception):
    ...


__all__ = ["InvalidType", "InvalidTemplateCall", "InvalidDispatchType"]



from enum import Enum
from typing import Dict, List, Optional

from hera.shared._base_model import BaseModel as BaseModel

from ...apimachinery.pkg.api import resource as resource
from ...apimachinery.pkg.apis.meta import v1 as v1
from ...apimachinery.pkg.util import intstr as intstr

class AWSElasticBlockStoreVolumeSource(BaseModel):
    fs_type: Optional[str]
    partition: Optional[int]
    read_only: Optional[bool]
    volume_id: str

class AzureDiskVolumeSource(BaseModel):
    caching_mode: Optional[str]
    disk_name: str
    disk_uri: str
    fs_type: Optional[str]
    kind: Optional[str]
    read_only: Optional[bool]

class AzureFileVolumeSource(BaseModel):
    read_only: Optional[bool]
    secret_name: str
    share_name: str

class Capabilities(BaseModel):
    add: Optional[List[str]]
    drop: Optional[List[str]]

class ConfigMapEnvSource(BaseModel):
    name: Optional[str]
    optional: Optional[bool]

class ConfigMapKeySelector(BaseModel):
    key: str
    name: Optional[str]
    optional: Optional[bool]

class ImagePullPolicy(Enum):
    always: str
    if_not_present: str
    never: str

class TerminationMessagePolicy(Enum):
    fallback_to_logs_on_error: str
    file: str

class Protocol(Enum):
    sctp: str
    tcp: str
    udp: str

class ContainerPort(BaseModel):
    container_port: int
    host_ip: Optional[str]
    host_port: Optional[int]
    name: Optional[str]
    protocol: Optional[Protocol]

class EventSource(BaseModel):
    component: Optional[str]
    host: Optional[str]

class ExecAction(BaseModel):
    command: Optional[List[str]]

class FCVolumeSource(BaseModel):
    fs_type: Optional[str]
    lun: Optional[int]
    read_only: Optional[bool]
    target_ww_ns: Optional[List[str]]
    wwids: Optional[List[str]]

class FlockerVolumeSource(BaseModel):
    dataset_name: Optional[str]
    dataset_uuid: Optional[str]

class GCEPersistentDiskVolumeSource(BaseModel):
    fs_type: Optional[str]
    partition: Optional[int]
    pd_name: str
    read_only: Optional[bool]

class GRPCAction(BaseModel):
    port: int
    service: Optional[str]

class GitRepoVolumeSource(BaseModel):
    directory: Optional[str]
    repository: str
    revision: Optional[str]

class GlusterfsVolumeSource(BaseModel):
    endpoints: str
    path: str
    read_only: Optional[bool]

class Scheme(Enum):
    http: str
    https: str

class HTTPHeader(BaseModel):
    name: str
    value: str

class HostAlias(BaseModel):
    hostnames: Optional[List[str]]
    ip: Optional[str]

class HostPathVolumeSource(BaseModel):
    path: str
    type: Optional[str]

class KeyToPath(BaseModel):
    key: str
    mode: Optional[int]
    path: str

class LocalObjectReference(BaseModel):
    name: Optional[str]

class NFSVolumeSource(BaseModel):
    path: str
    read_only: Optional[bool]
    server: str

class Operator(Enum):
    does_not_exist: str
    exists: str
    gt: str
    in_: str
    lt: str
    not_in: str

class NodeSelectorRequirement(BaseModel):
    key: str
    operator: Operator
    values: Optional[List[str]]

class NodeSelectorTerm(BaseModel):
    match_expressions: Optional[List[NodeSelectorRequirement]]
    match_fields: Optional[List[NodeSelectorRequirement]]

class ObjectFieldSelector(BaseModel):
    api_version: Optional[str]
    field_path: str

class ObjectReference(BaseModel):
    api_version: Optional[str]
    field_path: Optional[str]
    kind: Optional[str]
    name: Optional[str]
    namespace: Optional[str]
    resource_version: Optional[str]
    uid: Optional[str]

class Type(Enum):
    file_system_resize_pending: str
    resizing: str

class Phase(Enum):
    bound: str
    lost: str
    pending: str

class PersistentVolumeClaimVolumeSource(BaseModel):
    claim_name: str
    read_only: Optional[bool]

class PhotonPersistentDiskVolumeSource(BaseModel):
    fs_type: Optional[str]
    pd_id: str

class PodDNSConfigOption(BaseModel):
    name: Optional[str]
    value: Optional[str]

class PortworxVolumeSource(BaseModel):
    fs_type: Optional[str]
    read_only: Optional[bool]
    volume_id: str

class PreferredSchedulingTerm(BaseModel):
    preference: NodeSelectorTerm
    weight: int

class QuobyteVolumeSource(BaseModel):
    group: Optional[str]
    read_only: Optional[bool]
    registry: str
    tenant: Optional[str]
    user: Optional[str]
    volume: str

class RBDVolumeSource(BaseModel):
    fs_type: Optional[str]
    image: str
    keyring: Optional[str]
    monitors: List[str]
    pool: Optional[str]
    read_only: Optional[bool]
    secret_ref: Optional[LocalObjectReference]
    user: Optional[str]

class SELinuxOptions(BaseModel):
    level: Optional[str]
    role: Optional[str]
    type: Optional[str]
    user: Optional[str]

class ScaleIOVolumeSource(BaseModel):
    fs_type: Optional[str]
    gateway: str
    protection_domain: Optional[str]
    read_only: Optional[bool]
    secret_ref: LocalObjectReference
    ssl_enabled: Optional[bool]
    storage_mode: Optional[str]
    storage_pool: Optional[str]
    system: str
    volume_name: Optional[str]

class TypeModel(Enum):
    localhost: str
    runtime_default: str
    unconfined: str

class SeccompProfile(BaseModel):
    localhost_profile: Optional[str]
    type: TypeModel

class SecretEnvSource(BaseModel):
    name: Optional[str]
    optional: Optional[bool]

class SecretKeySelector(BaseModel):
    key: str
    name: Optional[str]
    optional: Optional[bool]

class SecretProjection(BaseModel):
    items: Optional[List[KeyToPath]]
    name: Optional[str]
    optional: Optional[bool]

class SecretVolumeSource(BaseModel):
    default_mode: Optional[int]
    items: Optional[List[KeyToPath]]
    optional: Optional[bool]
    secret_name: Optional[str]

class ServiceAccountTokenProjection(BaseModel):
    audience: Optional[str]
    expiration_seconds: Optional[int]
    path: str

class StorageOSVolumeSource(BaseModel):
    fs_type: Optional[str]
    read_only: Optional[bool]
    secret_ref: Optional[LocalObjectReference]
    volume_name: Optional[str]
    volume_namespace: Optional[str]

class Sysctl(BaseModel):
    name: str
    value: str

class Effect(Enum):
    no_execute: str
    no_schedule: str
    prefer_no_schedule: str

class OperatorModel(Enum):
    equal: str
    exists: str

class Toleration(BaseModel):
    effect: Optional[Effect]
    key: Optional[str]
    operator: Optional[OperatorModel]
    toleration_seconds: Optional[int]
    value: Optional[str]

class TypedLocalObjectReference(BaseModel):
    api_group: Optional[str]
    kind: str
    name: str

class VolumeDevice(BaseModel):
    device_path: str
    name: str

class VolumeMount(BaseModel):
    mount_path: str
    mount_propagation: Optional[str]
    name: str
    read_only: Optional[bool]
    sub_path: Optional[str]
    sub_path_expr: Optional[str]

class VsphereVirtualDiskVolumeSource(BaseModel):
    fs_type: Optional[str]
    storage_policy_id: Optional[str]
    storage_policy_name: Optional[str]
    volume_path: str

class WindowsSecurityContextOptions(BaseModel):
    gmsa_credential_spec: Optional[str]
    gmsa_credential_spec_name: Optional[str]
    host_process: Optional[bool]
    run_as_user_name: Optional[str]

class CSIVolumeSource(BaseModel):
    driver: str
    fs_type: Optional[str]
    node_publish_secret_ref: Optional[LocalObjectReference]
    read_only: Optional[bool]
    volume_attributes: Optional[Dict[str, str]]

class CephFSVolumeSource(BaseModel):
    monitors: List[str]
    path: Optional[str]
    read_only: Optional[bool]
    secret_file: Optional[str]
    secret_ref: Optional[LocalObjectReference]
    user: Optional[str]

class CinderVolumeSource(BaseModel):
    fs_type: Optional[str]
    read_only: Optional[bool]
    secret_ref: Optional[LocalObjectReference]
    volume_id: str

class ConfigMapProjection(BaseModel):
    items: Optional[List[KeyToPath]]
    name: Optional[str]
    optional: Optional[bool]

class ConfigMapVolumeSource(BaseModel):
    default_mode: Optional[int]
    items: Optional[List[KeyToPath]]
    name: Optional[str]
    optional: Optional[bool]

class EmptyDirVolumeSource(BaseModel):
    medium: Optional[str]
    size_limit: Optional[resource.Quantity]

class EnvFromSource(BaseModel):
    config_map_ref: Optional[ConfigMapEnvSource]
    prefix: Optional[str]
    secret_ref: Optional[SecretEnvSource]

class EventSeries(BaseModel):
    count: Optional[int]
    last_observed_time: Optional[v1.MicroTime]

class FlexVolumeSource(BaseModel):
    driver: str
    fs_type: Optional[str]
    options: Optional[Dict[str, str]]
    read_only: Optional[bool]
    secret_ref: Optional[LocalObjectReference]

class HTTPGetAction(BaseModel):
    host: Optional[str]
    http_headers: Optional[List[HTTPHeader]]
    path: Optional[str]
    port: intstr.IntOrString
    scheme: Optional[Scheme]

class ISCSIVolumeSource(BaseModel):
    chap_auth_discovery: Optional[bool]
    chap_auth_session: Optional[bool]
    fs_type: Optional[str]
    initiator_name: Optional[str]
    iqn: str
    iscsi_interface: Optional[str]
    lun: int
    portals: Optional[List[str]]
    read_only: Optional[bool]
    secret_ref: Optional[LocalObjectReference]
    target_portal: str

class NodeSelector(BaseModel):
    node_selector_terms: List[NodeSelectorTerm]

class PersistentVolumeClaimCondition(BaseModel):
    last_probe_time: Optional[v1.Time]
    last_transition_time: Optional[v1.Time]
    message: Optional[str]
    reason: Optional[str]
    status: str
    type: Type

class PersistentVolumeClaimStatus(BaseModel):
    access_modes: Optional[List[str]]
    allocated_resources: Optional[Dict[str, resource.Quantity]]
    capacity: Optional[Dict[str, resource.Quantity]]
    conditions: Optional[List[PersistentVolumeClaimCondition]]
    phase: Optional[Phase]
    resize_status: Optional[str]

class PodDNSConfig(BaseModel):
    nameservers: Optional[List[str]]
    options: Optional[List[PodDNSConfigOption]]
    searches: Optional[List[str]]

class PodSecurityContext(BaseModel):
    fs_group: Optional[int]
    fs_group_change_policy: Optional[str]
    run_as_group: Optional[int]
    run_as_non_root: Optional[bool]
    run_as_user: Optional[int]
    se_linux_options: Optional[SELinuxOptions]
    seccomp_profile: Optional[SeccompProfile]
    supplemental_groups: Optional[List[int]]
    sysctls: Optional[List[Sysctl]]
    windows_options: Optional[WindowsSecurityContextOptions]

class ResourceFieldSelector(BaseModel):
    container_name: Optional[str]
    divisor: Optional[resource.Quantity]
    resource: str

class ResourceRequirements(BaseModel):
    limits: Optional[Dict[str, resource.Quantity]]
    requests: Optional[Dict[str, resource.Quantity]]

class SecurityContext(BaseModel):
    allow_privilege_escalation: Optional[bool]
    capabilities: Optional[Capabilities]
    privileged: Optional[bool]
    proc_mount: Optional[str]
    read_only_root_filesystem: Optional[bool]
    run_as_group: Optional[int]
    run_as_non_root: Optional[bool]
    run_as_user: Optional[int]
    se_linux_options: Optional[SELinuxOptions]
    seccomp_profile: Optional[SeccompProfile]
    windows_options: Optional[WindowsSecurityContextOptions]

class ServicePort(BaseModel):
    app_protocol: Optional[str]
    name: Optional[str]
    node_port: Optional[int]
    port: int
    protocol: Optional[Protocol]
    target_port: Optional[intstr.IntOrString]

class TCPSocketAction(BaseModel):
    host: Optional[str]
    port: intstr.IntOrString

class DownwardAPIVolumeFile(BaseModel):
    field_ref: Optional[ObjectFieldSelector]
    mode: Optional[int]
    path: str
    resource_field_ref: Optional[ResourceFieldSelector]

class DownwardAPIVolumeSource(BaseModel):
    default_mode: Optional[int]
    items: Optional[List[DownwardAPIVolumeFile]]

class EnvVarSource(BaseModel):
    config_map_key_ref: Optional[ConfigMapKeySelector]
    field_ref: Optional[ObjectFieldSelector]
    resource_field_ref: Optional[ResourceFieldSelector]
    secret_key_ref: Optional[SecretKeySelector]

class Event(BaseModel):
    action: Optional[str]
    api_version: Optional[str]
    count: Optional[int]
    event_time: Optional[v1.MicroTime]
    first_timestamp: Optional[v1.Time]
    involved_object: ObjectReference
    kind: Optional[str]
    last_timestamp: Optional[v1.Time]
    message: Optional[str]
    metadata: v1.ObjectMeta
    reason: Optional[str]
    related: Optional[ObjectReference]
    reporting_component: Optional[str]
    reporting_instance: Optional[str]
    series: Optional[EventSeries]
    source: Optional[EventSource]
    type: Optional[str]

class LifecycleHandler(BaseModel):
    exec: Optional[ExecAction]
    http_get: Optional[HTTPGetAction]
    tcp_socket: Optional[TCPSocketAction]

class NodeAffinity(BaseModel):
    preferred_during_scheduling_ignored_during_execution: Optional[List[PreferredSchedulingTerm]]
    required_during_scheduling_ignored_during_execution: Optional[NodeSelector]

class PersistentVolumeClaimSpec(BaseModel):
    access_modes: Optional[List[str]]
    data_source: Optional[TypedLocalObjectReference]
    data_source_ref: Optional[TypedLocalObjectReference]
    resources: Optional[ResourceRequirements]
    selector: Optional[v1.LabelSelector]
    storage_class_name: Optional[str]
    volume_mode: Optional[str]
    volume_name: Optional[str]

class PersistentVolumeClaimTemplate(BaseModel):
    metadata: Optional[v1.ObjectMeta]
    spec: PersistentVolumeClaimSpec

class PodAffinityTerm(BaseModel):
    label_selector: Optional[v1.LabelSelector]
    namespace_selector: Optional[v1.LabelSelector]
    namespaces: Optional[List[str]]
    topology_key: str

class Probe(BaseModel):
    exec: Optional[ExecAction]
    failure_threshold: Optional[int]
    grpc: Optional[GRPCAction]
    http_get: Optional[HTTPGetAction]
    initial_delay_seconds: Optional[int]
    period_seconds: Optional[int]
    success_threshold: Optional[int]
    tcp_socket: Optional[TCPSocketAction]
    termination_grace_period_seconds: Optional[int]
    timeout_seconds: Optional[int]

class WeightedPodAffinityTerm(BaseModel):
    pod_affinity_term: PodAffinityTerm
    weight: int

class DownwardAPIProjection(BaseModel):
    items: Optional[List[DownwardAPIVolumeFile]]

class EnvVar(BaseModel):
    name: str
    value: Optional[str]
    value_from: Optional[EnvVarSource]

class EphemeralVolumeSource(BaseModel):
    volume_claim_template: Optional[PersistentVolumeClaimTemplate]

class Lifecycle(BaseModel):
    post_start: Optional[LifecycleHandler]
    pre_stop: Optional[LifecycleHandler]

class PersistentVolumeClaim(BaseModel):
    api_version: Optional[str]
    kind: Optional[str]
    metadata: Optional[v1.ObjectMeta]
    spec: Optional[PersistentVolumeClaimSpec]
    status: Optional[PersistentVolumeClaimStatus]

class PodAffinity(BaseModel):
    preferred_during_scheduling_ignored_during_execution: Optional[List[WeightedPodAffinityTerm]]
    required_during_scheduling_ignored_during_execution: Optional[List[PodAffinityTerm]]

class PodAntiAffinity(BaseModel):
    preferred_during_scheduling_ignored_during_execution: Optional[List[WeightedPodAffinityTerm]]
    required_during_scheduling_ignored_during_execution: Optional[List[PodAffinityTerm]]

class VolumeProjection(BaseModel):
    config_map: Optional[ConfigMapProjection]
    downward_api: Optional[DownwardAPIProjection]
    secret: Optional[SecretProjection]
    service_account_token: Optional[ServiceAccountTokenProjection]

class Affinity(BaseModel):
    node_affinity: Optional[NodeAffinity]
    pod_affinity: Optional[PodAffinity]
    pod_anti_affinity: Optional[PodAntiAffinity]

class Container(BaseModel):
    args: Optional[List[str]]
    command: Optional[List[str]]
    env: Optional[List[EnvVar]]
    env_from: Optional[List[EnvFromSource]]
    image: str
    image_pull_policy: Optional[ImagePullPolicy]
    lifecycle: Optional[Lifecycle]
    liveness_probe: Optional[Probe]
    name: Optional[str]
    ports: Optional[List[ContainerPort]]
    readiness_probe: Optional[Probe]
    resources: Optional[ResourceRequirements]
    security_context: Optional[SecurityContext]
    startup_probe: Optional[Probe]
    stdin: Optional[bool]
    stdin_once: Optional[bool]
    termination_message_path: Optional[str]
    termination_message_policy: Optional[TerminationMessagePolicy]
    tty: Optional[bool]
    volume_devices: Optional[List[VolumeDevice]]
    volume_mounts: Optional[List[VolumeMount]]
    working_dir: Optional[str]

class ProjectedVolumeSource(BaseModel):
    default_mode: Optional[int]
    sources: Optional[List[VolumeProjection]]

class Volume(BaseModel):
    aws_elastic_block_store: Optional[AWSElasticBlockStoreVolumeSource]
    azure_disk: Optional[AzureDiskVolumeSource]
    azure_file: Optional[AzureFileVolumeSource]
    cephfs: Optional[CephFSVolumeSource]
    cinder: Optional[CinderVolumeSource]
    config_map: Optional[ConfigMapVolumeSource]
    csi: Optional[CSIVolumeSource]
    downward_api: Optional[DownwardAPIVolumeSource]
    empty_dir: Optional[EmptyDirVolumeSource]
    ephemeral: Optional[EphemeralVolumeSource]
    fc: Optional[FCVolumeSource]
    flex_volume: Optional[FlexVolumeSource]
    flocker: Optional[FlockerVolumeSource]
    gce_persistent_disk: Optional[GCEPersistentDiskVolumeSource]
    git_repo: Optional[GitRepoVolumeSource]
    glusterfs: Optional[GlusterfsVolumeSource]
    host_path: Optional[HostPathVolumeSource]
    iscsi: Optional[ISCSIVolumeSource]
    name: str
    nfs: Optional[NFSVolumeSource]
    persistent_volume_claim: Optional[PersistentVolumeClaimVolumeSource]
    photon_persistent_disk: Optional[PhotonPersistentDiskVolumeSource]
    portworx_volume: Optional[PortworxVolumeSource]
    projected: Optional[ProjectedVolumeSource]
    quobyte: Optional[QuobyteVolumeSource]
    rbd: Optional[RBDVolumeSource]
    scale_io: Optional[ScaleIOVolumeSource]
    secret: Optional[SecretVolumeSource]
    storageos: Optional[StorageOSVolumeSource]
    vsphere_volume: Optional[VsphereVirtualDiskVolumeSource]



from typing import List, Optional, Union

from hera.workflows.env import _BaseEnv
from hera.workflows.env_from import _BaseEnvFrom
from hera.workflows.models import (
    EnvFromSource,
    EnvVar,
    ImagePullPolicy,
    ResourceRequirements,
    UserContainer as _ModelUserContainer,
)
from hera.workflows.resources import Resources
from hera.workflows.volume import _BaseVolume


class UserContainer(_ModelUserContainer):
    env: Optional[List[Union[_BaseEnv, EnvVar]]] = None  # type: ignore[assignment]
    env_from: Optional[List[Union[_BaseEnvFrom, EnvFromSource]]] = None  # type: ignore[assignment]
    image_pull_policy: Optional[Union[str, ImagePullPolicy]] = None  # type: ignore[assignment]
    resources: Optional[Union[Resources, ResourceRequirements]] = None  # type: ignore[assignment]
    volumes: Optional[List[_BaseVolume]] = None

    def build(self) -> _ModelUserContainer:
        return _ModelUserContainer(
            args=self.args,
            command=self.command,
            env=self.env,
            env_from=self.env_from,
            image=self.image,
            image_pull_policy=self.image_pull_policy,
            lifecycle=self.lifecycle,
            liveness_probe=self.liveness_probe,
            mirror_volume_mounts=self.mirror_volume_mounts,
            name=self.name,
            ports=self.ports,
            readiness_probe=self.readiness_probe,
            resources=self.resources,
            security_context=self.security_context,
            startup_probe=self.startup_probe,
            stdin=self.stdin,
            stdin_once=self.stdin_once,
            termination_message_path=self.termination_message_path,
            termination_message_policy=self.termination_message_policy,
            tty=self.tty,
            volume_devices=self.volume_devices,
            volume_mounts=None if self.volumes is None else [v._build_volume_mount() for v in self.volumes],
            working_dir=self.working_dir,
        )


__all__ = ["UserContainer"]



"""Hera classes
"""
# [DO NOT EDIT MANUALLY]
# Auto-generated by Hera via `make init-files`
# In order to add objects to the hera.workflows namespace
# add them to the __all__ list in the relevant module.
# Hera submodules should not use `from hera.workflows import X`
# themselves, as it introduces a circular dependency.

from hera.workflows.archive import ArchiveStrategy, NoneArchiveStrategy, TarArchiveStrategy, ZipArchiveStrategy
from hera.workflows.artifact import (
    Artifact,
    ArtifactoryArtifact,
    AzureArtifact,
    GCSArtifact,
    GitArtifact,
    HDFSArtifact,
    HTTPArtifact,
    OSSArtifact,
    RawArtifact,
    S3Artifact,
)
from hera.workflows.container import Container
from hera.workflows.container_set import ContainerNode, ContainerSet
from hera.workflows.cron_workflow import CronWorkflow
from hera.workflows.dag import DAG
from hera.workflows.data import Data
from hera.workflows.env import ConfigMapEnv, Env, FieldEnv, ResourceEnv, SecretEnv
from hera.workflows.env_from import ConfigMapEnvFrom, SecretEnvFrom
from hera.workflows.exceptions import InvalidDispatchType, InvalidTemplateCall, InvalidType
from hera.workflows.http_template import HTTP
from hera.workflows.operator import Operator
from hera.workflows.parameter import Parameter
from hera.workflows.resource import Resource
from hera.workflows.resources import Resources
from hera.workflows.retry_strategy import RetryPolicy, RetryStrategy
from hera.workflows.script import Script, script
from hera.workflows.service import WorkflowsService
from hera.workflows.steps import Parallel, Step, Steps
from hera.workflows.suspend import Suspend
from hera.workflows.task import Task, TaskResult
from hera.workflows.user_container import UserContainer
from hera.workflows.volume import (
    AccessMode,
    AWSElasticBlockStoreVolumeVolume,
    AzureDiskVolumeVolume,
    AzureFileVolumeVolume,
    CephFSVolumeVolume,
    CinderVolume,
    ConfigMapVolume,
    CSIVolume,
    DownwardAPIVolume,
    EmptyDirVolume,
    EphemeralVolume,
    ExistingVolume,
    FCVolume,
    FlexVolume,
    FlockerVolume,
    GCEPersistentDiskVolume,
    GitRepoVolume,
    GlusterfsVolume,
    HostPathVolume,
    ISCSIVolume,
    NFSVolume,
    PhotonPersistentDiskVolume,
    PortworxVolume,
    ProjectedVolume,
    QuobyteVolume,
    RBDVolume,
    ScaleIOVolume,
    SecretVolume,
    StorageOSVolume,
    Volume,
    VsphereVirtualDiskVolume,
)
from hera.workflows.workflow import Workflow
from hera.workflows.workflow_status import WorkflowStatus
from hera.workflows.workflow_template import WorkflowTemplate

__all__ = [
    "AWSElasticBlockStoreVolumeVolume",
    "AccessMode",
    "ArchiveStrategy",
    "Artifact",
    "ArtifactoryArtifact",
    "AzureArtifact",
    "AzureDiskVolumeVolume",
    "AzureFileVolumeVolume",
    "CSIVolume",
    "CephFSVolumeVolume",
    "CinderVolume",
    "ConfigMapEnv",
    "ConfigMapEnvFrom",
    "ConfigMapVolume",
    "Container",
    "ContainerNode",
    "ContainerSet",
    "CronWorkflow",
    "DAG",
    "Data",
    "DownwardAPIVolume",
    "EmptyDirVolume",
    "Env",
    "EphemeralVolume",
    "ExistingVolume",
    "FCVolume",
    "FieldEnv",
    "FlexVolume",
    "FlockerVolume",
    "GCEPersistentDiskVolume",
    "GCSArtifact",
    "GitArtifact",
    "GitRepoVolume",
    "GlusterfsVolume",
    "HDFSArtifact",
    "HTTP",
    "HTTPArtifact",
    "HostPathVolume",
    "ISCSIVolume",
    "InvalidDispatchType",
    "InvalidTemplateCall",
    "InvalidType",
    "NFSVolume",
    "NoneArchiveStrategy",
    "OSSArtifact",
    "Operator",
    "Parallel",
    "Parameter",
    "PhotonPersistentDiskVolume",
    "PortworxVolume",
    "ProjectedVolume",
    "QuobyteVolume",
    "RBDVolume",
    "RawArtifact",
    "Resource",
    "ResourceEnv",
    "Resources",
    "RetryPolicy",
    "RetryStrategy",
    "S3Artifact",
    "ScaleIOVolume",
    "Script",
    "SecretEnv",
    "SecretEnvFrom",
    "SecretVolume",
    "Step",
    "Steps",
    "StorageOSVolume",
    "Suspend",
    "TarArchiveStrategy",
    "Task",
    "TaskResult",
    "UserContainer",
    "Volume",
    "VsphereVirtualDiskVolume",
    "Workflow",
    "WorkflowStatus",
    "WorkflowTemplate",
    "WorkflowsService",
    "ZipArchiveStrategy",
    "script",
]
